/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model;

import java.io.Serializable;
import java.util.LinkedList;

/**
 * Class Version is base class for classes representing a version.
 */
public abstract class Version implements Comparable<Version>, Serializable {
	private static final long serialVersionUID = 1L;
	private static LinkedList<VersionFactory> repos;
	static {
		repos = new LinkedList<VersionFactory>();
        try {
            Class.forName("dk.vajhoej.model.version.StringVersionFactory");
        } catch (ClassNotFoundException e) {
            // ignore
        }
        try {
            Class.forName("dk.vajhoej.model.version.NoVersionFactory");
        } catch (ClassNotFoundException e) {
            // ignore
        }
        try {
            Class.forName("dk.vajhoej.model.version.OneNumberVersionFactory");
        } catch (ClassNotFoundException e) {
            // ignore
        }
        try {
            Class.forName("dk.vajhoej.model.version.TwoNumberVersionFactory");
        } catch (ClassNotFoundException e) {
            // ignore
        }
		try {
			Class.forName("dk.vajhoej.model.version.ThreeNumberVersionFactory");
		} catch (ClassNotFoundException e) {
			// ignore
		}
		try {
			Class.forName("dk.vajhoej.model.version.FourNumberVersionFactory");
		} catch (ClassNotFoundException e) {
			// ignore
		}
	}
	/**
	 * Compare to other version.
	 * @param v other version
	 * @return negative, zero or positive according to comparison
	 */
	public abstract int compareTo(Version v);
	@Override
	public abstract String toString();
    /**
     * Bump least significant part of version.
     * @return new version
     */
	public abstract Version bumpLeast();
    /**
     * Bump most significant part of version.
     * @return new version
     */
	public abstract Version bumpMost();
    /**
     * Bump second least significant part of version.
     * @return new version
     */
	public abstract Version bumpSecondLeast();
    /**
     * Bump second most significant part of version.
     * @return new version
     */
	public abstract Version bumpSecondMost();
	@Override
	public boolean equals(Object o) {
		if(getClass().equals(o.getClass())) {
			if(toString().equals(o.toString())) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	@Override
	public int hashCode() {
		return toString().hashCode();
	}
	/**
	 * Parse string with version.
	 * @param s string with version
	 * @return version
	 * @throws ModelException if problem
	 */
	public static Version parse(String s) throws ModelException {
		for(VersionFactory vf : repos) {
			Version v = vf.create(s);
			if(v != null)  {
				return v;
			}
		}
		throw new ModelException("No matching version scheme found");
	}
	/**
	 * Register version factory.
	 * @param vf version factory
	 */
	public static void registerFactory(VersionFactory vf) {
		repos.addFirst(vf);
	}
}
