/*
 * Copyright 2011 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;

/**
 * Class AppLookAndFeel contains information about a look and feel plus theme.
 */
public class AppLookAndFeel {
    private String lafClassName;
    private String lafDisplayName;
    private String themeClassName;
    private String themeDisplayName;
    /**
     * Create instance of AppLookAndFeel.
     * @param lafClassName L&F class name
     * @param lafDisplayName L&F display name
     * @param themeClassName theme class name
     * @param themeDisplayName theme display name
     */
    public AppLookAndFeel(String lafClassName, String lafDisplayName, String themeClassName, String themeDisplayName) {
        super();
        this.lafClassName = lafClassName;
        this.lafDisplayName = lafDisplayName;
        this.themeClassName = themeClassName;
        this.themeDisplayName = themeDisplayName;
    }
    /**
     * Get L&F class name.
     * @return L&F class name
     */
    public String getLafClassName() {
        return lafClassName;
    }
    /**
     * Set L&F class name.
     * @param lafClassName L&F class name
     */
    public void setLafClassName(String lafClassName) {
        this.lafClassName = lafClassName;
    }
    /**
     * Get L&F display name.
     * @return L&F display name
     */
    public String getLafDisplayName() {
        return lafDisplayName;
    }
    /**
     * Set L&F class name.
     * @param lafDisplayName L&F class name
     */
    public void setLafDisplayName(String lafDisplayName) {
        this.lafDisplayName = lafDisplayName;
    }
    /**
     * Get theme class name.
     * @return theme class name
     */
    public String getThemeClassName() {
        return themeClassName;
    }
    /**
     * Set theme class name.
     * @param themeClassName theme class name
     */
    public void setThemeClassName(String themeClassName) {
        this.themeClassName = themeClassName;
    }
    /**
     * Get theme display name.
     * @return theme display name
     */
    public String getThemeDisplayName() {
        return themeDisplayName;
    }
    /**
     * Set theme display name.
     * @param themeDisplayName theme display name
     */
    public void setThemeDisplayName(String themeDisplayName) {
        this.themeDisplayName = themeDisplayName;
    }
    // basic
    @Override
    public boolean equals(Object o) {
        return o != null &&
               o instanceof AppLookAndFeel &&
               ((AppLookAndFeel)o).lafClassName.equals(lafClassName) &&
               (((AppLookAndFeel)o).lafClassName == lafClassName || ((AppLookAndFeel)o).lafClassName.equals(lafClassName));
    }
    @Override
    public int hashCode() {
        return lafClassName.hashCode();
    }
    @Override
    public String toString() {
        return "L&F:" + lafDisplayName;
    }
    /**
     * Lookup available L&F.
     * @return available L&F
     */
    public static List<AppLookAndFeel> lookup() {
        List<AppLookAndFeel> res = new ArrayList<AppLookAndFeel>();
        for(LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
            if(info.getClassName().equals("javax.swing.plaf.metal.MetalLookAndFeel")) {
                res.add(new AppLookAndFeel(info.getClassName(), info.getName(), "javax.swing.plaf.metal.DefaultMetalTheme", "Default"));
                res.add(new AppLookAndFeel(info.getClassName(), info.getName(), "javax.swing.plaf.metal.OceanTheme", "Ocean"));
                res.add(new AppLookAndFeel(info.getClassName(), info.getName(), "dk.vajhoej.model.app.AppMetalTheme", "Customizable"));
            } else {
                res.add(new AppLookAndFeel(info.getClassName(), info.getName(), null, null));
            }
        }
        List<AppLookAndFeel> trylist = new ArrayList<AppLookAndFeel>();
        trylist.add(new AppLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel", "Windows", null, null));
        trylist.add(new AppLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel", "Motif", null, null));
        trylist.add(new AppLookAndFeel("com.sun.java.swing.plaf.gtk.GTKLookAndFeel", "GTK", null, null));
        trylist.add(new AppLookAndFeel("com.sun.java.swing.plaf.mac.MacLookAndFeel", "Mac", null, null));
        trylist.add(new AppLookAndFeel("javax.swing.plaf.nimbus.NimbusLookAndFeel", "Nimbus", null, null));
        trylist.add(new AppLookAndFeel("com.nilo.plaf.nimrod.NimRODLookAndFeel", "NimROD", null, null));
        trylist.add(new AppLookAndFeel("org.fife.plaf.Office2003.Office2003LookAndFeel", "Office 2003", null, null));
        trylist.add(new AppLookAndFeel("org.fife.plaf.OfficeXP.OfficeXPLookAndFeel", "Office XP", null, null));
        trylist.add(new AppLookAndFeel("org.fife.plaf.VisualStudio2005.VisualStudio2005LookAndFeel", "VS 2005", null, null));
        trylist.add(new AppLookAndFeel("com.digitprop.tonic.TonicLookAndFeel", "Tonic", null, null));
        trylist.add(new AppLookAndFeel("com.easynth.lookandfeel.EaSynthLookAndFeel", "EaSynth", null, null));
        trylist.add(new AppLookAndFeel("com.petersoft.white.PetersoftWhiteLookAndFeel", "Petersoft White", null, null));
        for(AppLookAndFeel lf : trylist) {
            if(!res.contains(lf)) {
                try {
                    Class.forName(lf.getLafClassName());
                    res.add(lf);
                } catch (ClassNotFoundException e) {
                    // nothing
                }
            }
        }
        return Collections.unmodifiableList(res);
    }
}
