/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.Persister;
import dk.vajhoej.model.SeparatePersistable;
import dk.vajhoej.model.core.BoxPalette;
import dk.vajhoej.model.core.ComponentPalette;
import dk.vajhoej.model.core.ConnectionPalette;
import dk.vajhoej.model.core.ExtModel;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;
import dk.vajhoej.model.persist.XmlPersister;

/**
 * Class Control reads and writes setup XML files.
 */
public class Control {
    /**
     * Read setup XML file.
     * @param fnm file name
     * @return setup data
     * @throws ModelException if problem
     */
	public static Data load(String fnm) throws ModelException {
		Persister p = new XmlPersister();
		Setup s = (Setup)p.load(fnm, Setup.class);
		Persister p2 = null;
		try {
			p2 = (Persister)Class.forName(s.getPersister()).newInstance();
			p2.setParams(s.getPersisterParams());
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		AppLookAndFeel laf = new AppLookAndFeel(s.getLaf().get(0), s.getLaf().get(1),
		                                        s.getLaf().size() > 3 ? s.getLaf().get(2) : null, s.getLaf().size() > 3 ? s.getLaf().get(3) : null);
		int w = s.getW();
		int h = s.getH();
		String parent;
        try {
            parent = (new File(fnm)).getCanonicalFile().getParent();
        } catch (IOException e) {
            throw new ModelException("Error in filename : " + fnm, e);
        }
		AllGraphicConfig pres = (AllGraphicConfig)p2.loadSmart(s.getPresentation(), AllGraphicConfig.class, parent);
		Map<String, Boolean> show = new HashMap<String, Boolean>();
        int ix;
		List<BoxPalette> boxes = new ArrayList<BoxPalette>();
        ix = 0;
		for(String b : s.getBoxes()) {
			boxes.add((BoxPalette)p2.loadSmart(b, BoxPalette.class, parent));
            show.put(b.replace("*", parent), s.getShowBoxes().get(ix++));
		}
		List<ComponentPalette> components = new ArrayList<ComponentPalette>();
        ix = 0;
		for(String c : s.getComponents()) {
			components.add((ComponentPalette)p2.loadSmart(c, ComponentPalette.class, parent));
            show.put(c.replace("*", parent), s.getShowComponents().get(ix++));
		}
		List<ConnectionPalette> connections = new ArrayList<ConnectionPalette>();
        ix = 0;
		for(String c : s.getConnections()) {
			connections.add((ConnectionPalette)p2.loadSmart(c, ConnectionPalette.class, parent));
            show.put(c.replace("*", parent), s.getShowConnections().get(ix++));
		}
		ExtModel model = (ExtModel)p2.loadSmart(s.getModel(), ExtModel.class, parent);
		return new Data(p2,
		                laf,
		                w,
		                h,
				        pres,
				        boxes,
				        components,
				        connections,
				        show,
				        model);
	}
	/**
	 * Write setup XML file.
	 * @param data setup data
	 * @param fnm file name
	 * @throws ModelException if problem
	 */
	public static void save(Data data, String fnm) throws ModelException {
	    String parent = (new File(fnm)).getParent();
		Setup2 s = new Setup2();
		s.setPresentation(data.getConfig().getStore().replace(parent, "*"));
		s.setPersister(data.getPersist().getClass().getName());
        s.getLaf().add(data.getLaf().getLafClassName());
        s.getLaf().add(data.getLaf().getLafDisplayName());
        if(data.getLaf().getThemeClassName() != null) {
            s.getLaf().add(data.getLaf().getThemeClassName());
        }
        if(data.getLaf().getThemeDisplayName() != null) {
            s.getLaf().add(data.getLaf().getThemeDisplayName());
        }
        s.setW(data.getWidth());
        s.setH(data.getHeight());
		s.setModel(data.getModel().getStore().replace(parent, "*"));
		for(BoxPalette bp : data.getBoxPalette()) {
			s.getBoxes().add(bp.getStore().replace(parent, "*"));
            s.getShowBoxes().add(data.getShowPalette().get(bp.getStore()));
		}
		for(ComponentPalette cp : data.getComponentPalette()) {
			s.getComponents().add(cp.getStore().replace(parent, "*"));
            s.getShowComponents().add(data.getShowPalette().get(cp.getStore()));
		}
		for(ConnectionPalette cp : data.getConnectionPalette()) {
			s.getConnections().add(cp.getStore().replace(parent, "*"));
            s.getShowConnections().add(data.getShowPalette().get(cp.getStore()));
		}
        Persister p = new XmlPersister();
		p.save(s, fnm);
	}
	/**
     * Write setup XML file and write all the rest of the data to the
     * designated storage..
     * @param data setup data
     * @param fnm file name
     * @throws ModelException if problem
	 */
	public static void saveAll(Data data, String fnm) throws ModelException {
	    save(data, fnm);
        Persister p = data.getPersist();
        p.save(data.getConfig());
        p.save(data.getModel());
        for(BoxPalette bp : data.getBoxPalette()) {
            p.save(bp);
        }
        for(ComponentPalette cp : data.getComponentPalette()) {
            p.save(cp);
        }
        for(ConnectionPalette cp : data.getConnectionPalette()) {
            p.save(cp);
        }
	}
	private static void replaceStore(SeparatePersistable sp, String s1, String s2) {
	    sp.setStore(sp.getStore().replace(s1,s2));
	}
	/**
	 * Replace in all storage locations.
	 * @param data setup data
	 * @param s1 string to search for
	 * @param s2 string to replace with
	 */
	public static void replaceStore(Data data, String s1, String s2) {
        replaceStore(data.getConfig(), s1, s2);
        replaceStore(data.getModel(), s1, s2);
        for(BoxPalette bp : data.getBoxPalette()) {
            replaceStore(bp, s1, s2);
        }
        for(ComponentPalette cp : data.getComponentPalette()) {
            replaceStore(cp, s1, s2);
        }
        for(ConnectionPalette cp : data.getConnectionPalette()) {
            replaceStore(cp, s1, s2);
        }
	}
}
