/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.app;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.PersistableBase;
import dk.vajhoej.model.SeparatePersistable;

/**
 * Class Setup is the in memory representation of a setup XML file.
 */
public class Setup extends PersistableBase implements SeparatePersistable {
	private static final long serialVersionUID = 1L;
	protected String presentation;
	protected String persister;
	protected String persisterParams;
	protected List<String> laf;
    protected int w;
	protected int h;
	protected String model;
	protected List<String> boxes;
	protected List<String> components;
	protected List<String> connections;
    protected List<Boolean> showBoxes;
    protected List<Boolean> showComponents;
    protected List<Boolean> showConnections;
	/**
	 * Create instance of Setup.
	 */
	public Setup() {
		super(null);
		laf = new ArrayList<String>();
		boxes = new ArrayList<String>();
		components = new ArrayList<String>();
		connections = new ArrayList<String>();
        showBoxes = new ArrayList<Boolean>();
        showComponents = new ArrayList<Boolean>();
        showConnections = new ArrayList<Boolean>();
	}
	/**
	 * Get name of presentation file (with all graphic configuration).
	 * @return name of presentation file
	 */
	public String getPresentation() {
		return presentation;
	}
	/**
	 * Set name of presentation file (with all graphic configuration).
	 * @param presentation name of presentation file
	 */
	public void setPresentation(String presentation) {
		this.presentation = presentation;
	}
	/**
	 * Get class name of persister.
	 * @return class name of persister
	 */
	public String getPersister() {
		return persister;
	}
	/**
	 * Set class name of persister.
	 * @param persister class name of persister
	 */
	public void setPersister(String persister) {
		this.persister = persister;
	}
    /**
     * Get persister parameters.
     * @return persister parameters
     */
    public String getPersisterParams() {
        return persisterParams;
    }
    /**
     * Set persister parameters.
     * @param persisterParams persister parameters
     */
    public void setPersisterParams(String persisterParams) {
        this.persisterParams = persisterParams;
    }
    /**
     * Get L&F.
     * @return L&F
     */
    public List<String> getLaf() {
        return laf;
    }
    /**
     * Get width.
     * @return width
     */
    public int getW() {
        return w;
    }
    /**
     * Set width.
     * @param w width
     */
    public void setW(int w) {
        this.w = w;
    }
    /**
     * Get height.
     * @return height
     */
    public int getH() {
        return h;
    }
    /**
     * Set height.
     * @param h height
     */
    public void setH(int h) {
        this.h = h;
    }
	/**
	 * Get name of model file.
	 * @return name of model file
	 */
	public String getModel() {
		return model;
	}
	/**
	 * Set name of model file.
	 * @param model name of model file
	 */
	public void setModel(String model) {
		this.model = model;
	}
    /**
     * Get list of names of files with box palettes.
     * @return list of names of files with box palettes
     */
	public List<String> getBoxes() {
		return boxes;
	}
    /**
     * Get list of names of files with component palettes.
     * @return list of names of files with component palettes
     */
	public List<String> getComponents() {
		return components;
	}
    /**
     * Get list of names of files with connection palettes.
     * @return list of names of files with connection palettes
     */
	public List<String> getConnections() {
		return connections;
	}
    /**
     * Get list of show status for box palettes.
     * @return list of show status
     */
    public List<Boolean> getShowBoxes() {
        return showBoxes;
    }
    /**
     * Get list of show status for component palettes.
     * @return list of show status
     */
    public List<Boolean> getShowComponents() {
        return showComponents;
    }
    /**
     * Get list of show status for connection palettes.
     * @return list of show status
     */
    public List<Boolean> getShowConnections() {
        return showConnections;
    }
	// Persistable
	public String getId() {
		return "Single";
	}
	public String getType() {
		return "setup";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("presentation", presentation);
		res.put("persister", persister);
		res.put("persisterParams", (persisterParams != null) ? persisterParams : "");
		if(laf.size() == 2) {
            res.put("laf", laf.get(0) + " " + laf.get(1));
		} else if(laf.size() == 4) {
            res.put("laf", laf.get(0) + " " + laf.get(1) + " " + laf.get(2) + " " + laf.get(3));
		} else {
		    throw new ModelRuntimeException("Invalid L&F in setup file");
		}
        res.put("w", Integer.toString(w));
        res.put("h", Integer.toString(h));
		res.put("model", model);
		StringBuilder sb;
		sb = new StringBuilder();
		for(String b : boxes) {
			sb.append(" ");
			sb.append(b);
		}
		res.put("boxes", sb.substring(1));
		sb = new StringBuilder();
		for(String c : components) {
			sb.append(" ");
			sb.append(c);
		}
		res.put("components", sb.substring(1));
		sb = new StringBuilder();
		for(String c : connections) {
			sb.append(" ");
			sb.append(c);
		}
		res.put("connections", sb.substring(1));
        sb = new StringBuilder();
        for(Boolean b : showBoxes) {
            sb.append(" ");
            sb.append(b.toString());
        }
        res.put("showBoxes", sb.substring(1));
        sb = new StringBuilder();
        for(Boolean c : showComponents) {
            sb.append(" ");
            sb.append(c.toString());
        }
        res.put("showComponents", sb.substring(1));
        sb = new StringBuilder();
        for(Boolean c : showConnections) {
            sb.append(" ");
            sb.append(c.toString());
        }
        res.put("showConnections", sb.substring(1));
        AppMetalThemeData theme = AppMetalThemeData.getInstance();
        res.put("metalTheme", Integer.toHexString(theme.getBorder().getRGB()).substring(2) + " " +
                              Integer.toHexString(theme.getMarked().getRGB()).substring(2) + " " +
                              Integer.toHexString(theme.getBackground().getRGB()).substring(2));
		return res;
	}
	public List<List<Persistable>> getChildren() {
		return new ArrayList<List<Persistable>>();
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		presentation = attr.get("presentation");
		persister = attr.get("persister");
		persisterParams = (attr.get("persisterParams") != null) ? attr.get("persister") : "";
		if(attr.get("laf") != null) {
		    for(String s : attr.get("laf").split(" ")) {
		        laf.add(s);
		    }
		} else {
            laf.add("javax.swing.plaf.metal.MetalLookAndFeel");
            laf.add("Metal");
		}
        if(attr.get("w") != null) {
            w = Integer.parseInt(attr.get("w"));
        } else {
            w = 0;
        }
        if(attr.get("h") != null) {
            h = Integer.parseInt(attr.get("h"));
        } else {
            h = 0;
        }
		model = attr.get("model");
		boxes = Arrays.asList(attr.get("boxes").split(" "));
		components = Arrays.asList(attr.get("components").split(" "));
		connections = Arrays.asList(attr.get("connections").split(" "));
        if(attr.get("showBoxes") != null) {
            for(String s : attr.get("showBoxes").split(" ")) {
                showBoxes.add(Boolean.parseBoolean(s));
            }
        } else {
            for(int i = 0; i < boxes.size(); i++) {
                showBoxes.add(true);
            }
        }
        if(attr.get("showComponents") != null) {
            for(String s : attr.get("showComponents").split(" ")) {
                showComponents.add(Boolean.parseBoolean(s));
            }
        } else {
            for(int i = 0; i < components.size(); i++) {
                showComponents.add(true);
            }
        }
        if(attr.get("showConnections") != null) {
            for(String s : attr.get("showConnections").split(" ")) {
                showConnections.add(Boolean.parseBoolean(s));
            }
        } else {
            for(int i = 0; i < connections.size(); i++) {
                showConnections.add(true);
            }
        }
        if(attr.get("metalTheme") != null) {
            String[] cols = attr.get("metalTheme").split(" ");
            AppMetalThemeData theme = AppMetalThemeData.getInstance();
            theme.setBorder(Color.decode("0x" + cols[0]));
            theme.setMarked(Color.decode("0x" + cols[1]));
            theme.setBackground(Color.decode("0x" + cols[2]));
        }
	}
	public void setChildren(List<List<Persistable>> childs) throws ModelException {
	}
}
