/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import dk.vajhoej.model.Duplicatable;
import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.Util;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.Versioned;
import dk.vajhoej.model.version.StringVersion;

/**
 * Class Box represents a hardware box.
 */
public class Box extends Versioned implements Persistable, Duplicatable, Serializable {
	private static final long serialVersionUID = 1L;
	private String name;
	private String vendor;
	private String model;
	private List<Component> comps;
	private BoxType boxtype;
	private int cpu;
	private double mem;
	private double disk;
	private String id;
	/**
     * Create instance of Box.
	 */
	public Box() {
		this("", new StringVersion(""), "", "");
	}
	/**
     * Create instance of Box.
	 * @param name name
	 * @param version version
	 * @param vendor vendor
	 * @param model model
	 */
	public Box(String name, Version version, String vendor, String model) {
		this(name, version, vendor, model, BoxType.SINGLE_SYS, 0, 0, 0);
	}
	/**
	 * Create instance of Box.
     * @param name name
     * @param version version
     * @param vendor vendor
     * @param model model
	 * @param boxtype box type
	 * @param cpu CPU cores
	 * @param mem GB memory
	 * @param disk TB disk space
	 */
	public Box(String name, Version version, String vendor, String model, BoxType boxtype, int cpu, double mem, double disk) {
		super(version);
		this.name = name;
		this.vendor = vendor;
		this.model = model;
		comps = new ArrayList<Component>();
		this.boxtype = boxtype;
		this.cpu = cpu;
		this.mem = mem;
		this.disk = disk;
		id = "tmpbox#" + System.currentTimeMillis();
	}
	/**
	 * Get name.
	 * @return name
	 */
	public String getName() {
		return name;
	}
    /**
     * Set name.
     * @param name name
     */
    public void setName(String name) {
        this.name = name;
    }
	/**
	 * Get vendor.
	 * @return vendor
	 */
	public String getVendor() {
		return vendor;
	}
    /**
     * Set vendor.
     * @param vendor vendor
     */
    public void setVendor(String vendor) {
        this.vendor = vendor;
    }
	/**
	 * Get model.
	 * @return model
	 */
	public String getModel() {
		return model;
	}
    /**
     * Set model.
     * @param model model
     */
    public void setModel(String model) {
        this.model = model;
    }
	/**
	 * Get components.
	 * @return unmodifiable list of components 
	 */
	public List<Component> getComponents() {
		return Collections.unmodifiableList(comps);
	}
	/**
	 * Add component.
	 * @param c components
	 */
	public void addComponent(Component c) {
		comps.add(c);
	}
	/**
	 * Remove component. 
	 * @param c component
	 */
	public void removeComponent(Component c) {
		comps.remove(c);
	}
	/**
	 * Get box type.
	 * @return box type
	 */
	public BoxType getBoxtype() {
		return boxtype;
	}
	/**
	 * Get CPU.
	 * @return CPU cores
	 */
	public int getCpu() {
		return cpu;
	}
    /**
     * Set CPU.
     * @param cpu CPU cores
     */
    public void setCpu(int cpu) {
        this.cpu = cpu;
    }
	/**
	 * Get memory.
	 * @return GB memory
	 */
	public double getMem() {
		return mem;
	}
    /**
     * Set memory.
     * @param mem GB memory
     */
    public void setMem(double mem) {
        this.mem = mem;
    }
	/**
	 * Get disk space.
	 * @return TB disk
	 */
	public double getDisk() {
		return disk;
	}
    /**
     * Set disk space.
     * @param disk TB disk
     */
    public void setDisk(double disk) {
        this.disk = disk;
    }
	/**
	 * Set ID.
	 * @param id id
	 */
	public void setId(String id) {
	    this.id = id;
	}
	/**
	 * Set ID.
	 * @param id id
	 */
	public void setId(int id) {
	    setId("Box#" + id);
	}
	// Duplicatable
	public Box duplicate() {
		Box res = new Box(name, getVersion(), vendor, model, boxtype, cpu, mem, disk);
		for(Component c : comps) {
			res.addComponent(c.duplicate());
		}
		return res;
	}
	// Persistable:
	public String getId() {
        return id;
	}
	public String getType() {
		return "box";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("name", name);
		res.put("vendor", vendor);
		res.put("model", model);
		res.put("version", getVersion().toString());
		res.put("boxtype", boxtype.toString());
		if(boxtype == BoxType.SINGLE_SYS || boxtype == BoxType.MULTI_SYS) {
			res.put("cpu", Integer.toString(cpu));
		}
		if(boxtype == BoxType.SINGLE_SYS || boxtype == BoxType.MULTI_SYS ||
		   boxtype == BoxType.SINGLE_DISK || boxtype == BoxType.MULTI_SYS) {
			res.put("mem", Util.intOrDoubleToString(mem));
			res.put("disk", Util.intOrDoubleToString(disk));
		}
        res.put("id", id);
		return res;
	}
	public List<List<Persistable>> getChildren() {
		List<Persistable> res0 = new ArrayList<Persistable>();
		res0.addAll(comps);
		List<List<Persistable>> res = new ArrayList<List<Persistable>>();
		res.add(res0);
		return res;
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		setVersion(Version.parse(attr.get("version")));
		name = attr.get("name");
		vendor = attr.get("vendor");
		model = attr.get("model");
		boxtype = attr.get("boxtype") != null ? BoxType.parse(attr.get("boxtype")) : BoxType.SINGLE_SYS;
        try {
		    cpu = attr.get("cpu") != null ? Integer.parseInt(attr.get("cpu")) : 0;
        } catch(NumberFormatException ex) {
            throw new ModelException("'" + attr.get("cpu") + "' is not a valid value for Cpu");
        }
        try {
            mem = attr.get("mem") != null ? Double.parseDouble(attr.get("mem")) : 0;
        } catch(NumberFormatException ex) {
            throw new ModelException("'" + attr.get("mem") + "' is not a valid value for Mem");
        }
        try {
            disk = attr.get("disk") != null ? Double.parseDouble(attr.get("disk")) : 0;
        } catch(NumberFormatException ex) {
            throw new ModelException("'" + attr.get("disk") + "' is not a valid value for Disk");
        }
		id = attr.get("id");
	}
	public void setChildren(List<List<Persistable>> childs) {
		for(Persistable p : childs.get(0)) {
			comps.add((Component)p);
		}
	}
	// basic
	@Override
	public boolean equals(Object o) {
		if(o instanceof Box) {
			Box b = (Box)o;
			if(id.equals(b.id)) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	@Override
	public int hashCode() {
		return id.hashCode();
	}
	@Override
	public String toString() {
	    return "(" + name + "," + vendor + "," + model + "," + comps + "," + boxtype + "," + cpu + "," + mem + "," + disk + "," + id + ")";
	}
}
