/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import dk.vajhoej.model.Duplicatable;
import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.PaletteBase;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.SeparatePersistable;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.version.StringVersion;

/**
 * Class BoxPalette represents a palette of hardware box.
 */
public class BoxPalette extends PaletteBase implements SeparatePersistable, Duplicatable, Serializable {
	private static final long serialVersionUID = 1L;
	private String name;
	private List<Box> boxes;
	/**
	 * Create instance of BoxPalette.
	 */
	public BoxPalette() {
		this("", new StringVersion(""));
	}
	/**
     * Create instance of BoxPalette.
	 * @param name name
	 * @param version version
	 */
	public BoxPalette(String name, Version version) {
		super(version);
		this.name = name;
		boxes = new ArrayList<Box>();
	}
	/**
	 * Get name.
	 * @return name
	 */
	public String getName() {
		return name;
	}
	/**
	 * Get boxes.
	 * @return unmodifiable list of boxes
	 */
	public List<Box> getBoxes() {
		return Collections.unmodifiableList(boxes);
	}
	/**
	 * Add box.
	 * @param box box
	 */
	public void addBox(Box box) {
        boxes.add(box);
	}
	/**
	 * Remove box.
	 * @param box box
	 */
	public void removeBox(Box box) {
        boxes.remove(box);
	}
	// Duplicatable
	public BoxPalette duplicate() {
		BoxPalette res = new BoxPalette(name, getVersion());
		for(Box b : boxes) {
			res.addBox(b.duplicate());
		}
		return res;
	}
	// Persistable:
	public String getId() {
	    return name;
	}
	public String getType() {
		return "boxPalette";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("name", name);
		res.put("version", getVersion().toString());
		return res;
	}
	public List<List<Persistable>> getChildren() {
		List<Persistable> res0 = new ArrayList<Persistable>();
		res0.addAll(boxes);
		List<List<Persistable>> res = new ArrayList<List<Persistable>>();
		res.add(res0);
		return res;
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		name = attr.get("name");
		setVersion(Version.parse(attr.get("version")));
	}
	public void setChildren(List<List<Persistable>> childs) {
		for(Persistable p : childs.get(0)) {
			boxes.add((Box)p);
		}
	}
    // basic
    @Override
    public boolean equals(Object o) {
        if(o instanceof BoxPalette) {
            BoxPalette bp = (BoxPalette)o;
            if(name.equals(bp.name) && getVersion().equals(bp.getVersion())) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    @Override
    public int hashCode() {
        return (name + getVersion()).hashCode();
    }
    @Override
    public String toString() {
        return "(" + name + "," + boxes + ")";
    }
}
