/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import dk.vajhoej.model.Duplicatable;
import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.PaletteBase;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.SeparatePersistable;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.version.StringVersion;

/**
 * Class ComponentPalette represents a palette of software components.
 */
public class ComponentPalette extends PaletteBase implements SeparatePersistable, Duplicatable, Serializable {
	private static final long serialVersionUID = 1L;
	private String name;
	private List<Component> comps;
	/**
     * Create instance of ComponentPalette.
	 */
	public ComponentPalette() {
		this("", new StringVersion(""));
	}
	/**
     * Create instance of ComponentPalette.
	 * @param name name
	 * @param version version
	 */
	public ComponentPalette(String name, Version version) {
		super(version);
		this.name = name;
		comps = new ArrayList<Component>();
	}
	/**
	 * Get name.
	 * @return name
	 */
	public String getName() {
		return name;
	}
	/**
	 * Get components.
	 * @return unmodifiable list of components
	 */
	public List<Component> getComponents() {
		return Collections.unmodifiableList(comps);
	}
	public void addComponent(Component c) {
		comps.add(c);
	}
	// Duplicateable
	public ComponentPalette duplicate() {
		ComponentPalette res = new ComponentPalette(name, getVersion());
		for(Component c : comps) {
			res.addComponent(c.duplicate());
		}
		return res;
	}
	// Persistable:
    public String getId() {
        return name;
    }
	public String getType() {
		return "componentPalette";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("name", name);
		res.put("version", getVersion().toString());
		return res;
	}
	public List<List<Persistable>> getChildren() {
		List<Persistable> res0 = new ArrayList<Persistable>();
		res0.addAll(comps);
		List<List<Persistable>> res = new ArrayList<List<Persistable>>();
		res.add(res0);
		return res;
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		name = attr.get("name");
		setVersion(Version.parse(attr.get("version")));
	}
	public void setChildren(List<List<Persistable>> childs) {
		for(Persistable p : childs.get(0)) {
			comps.add((Component)p);
		}
	}
    // basic
    @Override
    public boolean equals(Object o) {
        if(o instanceof ComponentPalette) {
            ComponentPalette cp = (ComponentPalette)o;
            if(name.equals(cp.name) && getVersion().equals(cp.getVersion())) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    @Override
    public int hashCode() {
        return (name + getVersion()).hashCode();
    }
    @Override
    public String toString() {
        return "(" + name + "," + comps + ")";
    }
}
