/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

/**
 * Enum ComponentStatus specifies status of software component
 * regarding redundancy.
 * <br>
 * Semantics:
 * <table border="yes">
 * <tr>
 * <th>enum value</th>
 * <th>description</th>
 * </tr>
 * <tr>
 * <td>LOCAL</td>
 * <td>Component is only accessible local</td>
 * </tr>
 * <tr>
 * <td>REMOTE_SINGLE</td>
 * <td>Component is accessible remote but always as single instance</td>
 * </tr>
 * <tr>
 * <td>REMOTE_LOADSHARING</td>
 * <td>Component is accessible remote in loadsharing (active-active)
 * configurations</td>
 * </tr>
 * <tr>
 * <td>REMOTE_FAILOVER</td>
 * <td>Component is accessible remote in failover (active-passive)
 * configurations - this instance is active</td>
 * </tr>
 * <tr>
 * <td>REMOTE_FAILOVER_PASSIVE</td>
 * <td>Component is accessible remote in failover (active-passive)
 * configurations - this instance is passive</td>
 * </tr>
 * <tr>
 * <td>REMOTE_PASSTHROUGH</td>
 * <td>Component is accessible remote but single/loadsharing/failover
 * is determined by software components inside</td>
 * </tr>
 * </border>
 */
public enum ComponentStatus {
	LOCAL,
	REMOTE_SINGLE,
	REMOTE_LOADSHARING,
    REMOTE_FAILOVER,
    REMOTE_FAILOVER_PASSIVE,
	REMOTE_PASSTHROUGH;
	@Override
	public String toString() {
		switch(this) {
			case LOCAL:
				return "Local";
			case REMOTE_SINGLE:
				return "Single";
			case REMOTE_LOADSHARING:
				return "Loadsharing";
            case REMOTE_FAILOVER:
                return "Failover";
            case REMOTE_FAILOVER_PASSIVE:
                return "Failover passive";
			case REMOTE_PASSTHROUGH:
				return "Passthrough";
			default:
				return "Unknown";
		}
	}
	/**
	 * Parse string with component status.
	 * @param s string with component status
	 * @return component status
	 */
	public static ComponentStatus parse(String s) {
		for(ComponentStatus e : ComponentStatus.values()) {
			if(e.toString().equals(s)) {
				return e;
			}
		}
		return null;
	}
}
