/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.Duplicatable;
import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.Versioned;
import dk.vajhoej.model.version.StringVersion;

/**
 * Class Connection represents a connection between boxes.
 */
public class Connection extends Versioned implements Persistable, Duplicatable, Serializable {
	private static final long serialVersionUID = 1L;
	private String stack;
	private ConnectionType conntype;
	private String id;
	/**
     * Create instance of Connection.
	 */
	public Connection() {
		this(new StringVersion(""), "");
	}
	/**
     * Create instance of Connection.
	 * @param version version
	 * @param stack protocol stack
	 */
	public Connection(Version version, String stack) {
	    this(version, stack, ConnectionType.SINGLE);
	}
	/**
	 * Create instance of Connection.
	 * @param version version
	 * @param stack protocol stack
	 * @param conntype connection type
	 */
	public Connection(Version version, String stack, ConnectionType conntype) {
        super(version);
        this.stack = stack;
        this.conntype = conntype;
        id = "tmpconnection#" + System.currentTimeMillis();
	}
	/**
	 * Get protocol stack.
	 * @return protocol stack.
	 */
	public String getStack() {
		return stack;
	}
	/**
	 * Set protocol stack.
	 * @param stack protocol stack
	 */
	public void setStack(String stack) {
	    this.stack = stack;
	}
	/**
	 * Get connection type.
	 * @return connection type
	 */
	public ConnectionType getConntype() {
	    return conntype;
	}
    /**
     * Set ID.
     * @param id id
     */
    public void setId(String id) {
        this.id = id;
    }
    /**
     * Set ID.
     * @param id id
     */
    public void setId(int id) {
        setId("Connection#" + id);
    }
	// Duplicatable
	public Connection duplicate() {
		return new Connection(getVersion(), stack, conntype);
	}
	// Persistable:
    public String getId() {
        return id;
    }
	public String getType() {
		return "connection";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("version", getVersion().toString());
		res.put("stack", stack);
		res.put("conntype", conntype.toString());
		res.put("id", id);
		return res;
	}
	public List<List<Persistable>> getChildren() {
		return new ArrayList<List<Persistable>>();
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		setVersion(Version.parse(attr.get("version")));
		stack = attr.get("stack");
		conntype = attr.get("conntype") != null ? ConnectionType.parse(attr.get("conntype")) : ConnectionType.SINGLE; 
        id = attr.get("id");
	}
	public void setChildren(List<List<Persistable>> childs) {
	}
	// basic
	@Override
	public boolean equals(Object o) {
		if(o instanceof Connection) {
			Connection c = (Connection)o;
			if(id.equals(c.id)) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	@Override
	public int hashCode() {
		return id.hashCode();
	}
	@Override
	public String toString() {
	    return "(" + stack + "," + conntype + "," + id + ")";
	}
}
