/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import dk.vajhoej.model.Duplicatable;
import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.PaletteBase;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.SeparatePersistable;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.version.StringVersion;

/**
 * Class ConnectionPalette represents a palette of connections between boxes.
 */
public class ConnectionPalette extends PaletteBase implements SeparatePersistable, Duplicatable, Serializable {
	private static final long serialVersionUID = 1L;
	private String name;
	private List<Connection> connections;
	/**
     * Create instance of ConnectionPalette.
	 */
	public ConnectionPalette() {
		this("", new StringVersion(""));
	}
	/**
     * Create instance of ConnectionPalette.
	 * @param name name
	 * @param version version
	 */
	public ConnectionPalette(String name, Version version) {
		super(version);
		this.name = name;
		connections = new ArrayList<Connection>();
	}
	/**
	 * Get name.
	 * @return name
	 */
	public String getName() {
		return name;
	}
	/**
	 * Get connections.
	 * @return unmodifiable list of connections
	 */
	public List<Connection> getConnections() {
		return Collections.unmodifiableList(connections);
	}
    /**
     * Add connection.
     * @param con connection
     */
    public void addConnection(Connection con) {
        connections.add(con);
    }
    /**
     * Remove connection.
     * @param con connection
     */
    public void removeConnection(Connection con) {
        connections.remove(con);
    }
	// Duplicatable
	public ConnectionPalette duplicate() {
		ConnectionPalette res = new ConnectionPalette(name, getVersion());
		for(Connection c : connections) {
			res.addConnection(c.duplicate());
		}
		return res;
	}
	// Persistable:
    public String getId() {
        return name;
    }
	public String getType() {
		return "connection";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("name", name);
		res.put("version", getVersion().toString());
		return res;
	}
	public List<List<Persistable>> getChildren() {
		List<Persistable> res0 = new ArrayList<Persistable>();
		res0.addAll(connections);
		List<List<Persistable>> res = new ArrayList<List<Persistable>>();
		res.add(res0);
		return res;
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		name = attr.get("name");
		setVersion(Version.parse(attr.get("version")));
	}
	public void setChildren(List<List<Persistable>> childs) {
		for(Persistable p : childs.get(0)) {
			connections.add((Connection)p);
		}
	}
    // basic
    @Override
    public boolean equals(Object o) {
        if(o instanceof ConnectionPalette) {
            ConnectionPalette cp = (ConnectionPalette)o;
            if(name.equals(cp.name) && getVersion().equals(cp.getVersion())) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    @Override
    public int hashCode() {
        return (name + getVersion()).hashCode();
    }
    @Override
    public String toString() {
        return "(" + name + "," + connections + ")";
    }
}
