/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.SeparatePersistable;

/**
 * Class ExtModel represents an architectural model extended
 * with visual information.
 */
public class ExtModel implements SeparatePersistable {
	private Model real;
	private List<Location> locations;
	private int dimX;
	private int dimY;
	private String store;
	/**
     * Create instance of ExtModel.
	 */
	public ExtModel() {
		this(new Model(), new ArrayList<Location>(), 0, 0);
	}
	/**
     * Create instance of ExtModel.
	 * @param real the model
	 * @param locations locations of boxes and connections
	 * @param dimX grid width
	 * @param dimY grid height
	 */
	public ExtModel(Model real, List<Location> locations, int dimX, int dimY) {
		this.real = real;
		this.locations = locations;
		this.dimX = dimX;
		this.dimY = dimY;
		this.store = System.getProperty("user.dir") + System.getProperty("file.separator") + real.getName() + " " + real.getVersion() + ".xml";
	}
	/**
	 * Get the model.
	 * @return the model
	 */
	public Model getReal() {
		return real;
	}
	/**
	 * Get locations.
	 * @return list of locations
	 */
	public List<Location> getLocations() {
		return locations;
	}
	/**
	 * Set locations.
	 * @param locations list of locations
	 */
	public void setLocations(List<Location> locations) {
	    this.locations = locations;
	}
    /**
     * Get grid width.
     * @return grid width
     */
	public int getDimX() {
		return dimX;
	}
	/**
     * Set grid width.
     * @param dimX grid width
	 */
    public void setDimX(int dimX) {
        this.dimX = dimX;
    }
    /**
     * Get grid height.
     * @return grid height
     */
	public int getDimY() {
		return dimY;
	}
	/**
	 * Set grid height.
	 * @param dimY grid height
	 */
	public void setDimY(int dimY) {
		this.dimY = dimY;
	}
	// Persistable
	public String getId() {
		return real.getId();
	}
	public String getType() {
		return "extModel";
	}
	public String getStore() {
		return store;
	}
	public void setStore(String store) {
		this.store = store;
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = real.getAttributes();
		res.put("dimX", Integer.toString(dimX));
		res.put("dimY", Integer.toString(dimY));
		return res;
	}
	public List<List<Persistable>> getChildren() {
		List<List<Persistable>> res = real.getChildren();
		List<Persistable> xres = new ArrayList<Persistable>();
		xres.addAll(locations);
		res.add(xres);
		return res;
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		real.setAttributes(attr);
		dimX = Integer.parseInt(attr.get("dimX"));
		dimY = Integer.parseInt(attr.get("dimY"));
	}
	public void setChildren(List<List<Persistable>> childs) throws ModelException {
		for(Persistable p : childs.get(childs.size() - 1)) {
			locations.add((Location)p);
		}
		// make Model setChildren work with optional lists
		childs.remove(childs.size() - 1);
		real.setChildren(childs);
	}
	// basic
	@Override
	public String toString() {
	    return "(" + real + "," + locations + "," + dimX + "," + dimY + ")";
	}
}
