/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;

/**
 * Class Location represents a visual location.
 */
public class Location implements Persistable, Cloneable, Serializable {
	private static final long serialVersionUID = 1L;
	private int x;
	private int y;
	private String clazz;
	private String id;
	private int angle;
	private double hor;
	private double ver;
	/**
     * Create instance of location.
	 */
	public Location() {
		this(0, 0, "", "");
	}
	/**
     * Create instance of location.
	 * @param x x coordinate
	 * @param y y coordinate
	 * @param clazz Java class name
	 * @param id object id
	 */
	public Location(int x, int y, String clazz, String id) {
		this(x, y, clazz, id, 0);
	}
	/**
     * Create instance of location.
     * @param x x coordinate
     * @param y y coordinate
     * @param clazz Java class name
	 * @param angle rotation angle
	 */
	public Location(int x, int y, String clazz, String id, int angle) {
	    this(x, y, clazz, id, angle, 0.0, 0.0);
	}
    /**
     * Create instance of location.
     * @param x x coordinate
     * @param y y coordinate
     * @param clazz Java class name
     * @param angle rotation angle
     * @param hor relative horizontal move
     * @param ver relative vertical move
     */
    public Location(int x, int y, String clazz, String id, int angle, double hor, double ver) {
        this.x = x;
        this.y = y;
        this.clazz = clazz;
        this.id = id;
        this.angle = angle;
        this.hor = hor;
        this.ver = ver;
    }
	/**
	 * Get x coordinate.
	 * @return x coordinate
	 */
	public int getX() {
		return x;
	}
	/**
	 * Get y coordinate.
	 * @return y coordinate
	 */
	public int getY() {
		return y;
	}
	/**
	 * Get Java class name.
	 * @return Java class name
	 */
	public String getClazz() {
		return clazz;
	}
	/**
	 * Get object id.
	 * @return object id
	 */
	public String getId() {
		return id;
	}
	/**
	 * Get rotation angle.
	 * @return rotation angle
	 */
	public int getAngle() {
		return angle;
	}
    /**
     * Get relative horizontal move.
     * @return horizontal move
     */
	public double getHor() {
	    return hor;
	}
    /**
     * Get relative vertical move.
     * @return vertical move
     */
	public double getVer() {
	    return ver;
	}
	/**
	 * Move locations.
	 * @param dx delta x
	 * @param dy delta y
	 */
	public void move(int dx, int dy) {
	    x += dx;
	    y += dy;
	}
	// Cloneable
	@Override
	public Location clone() {
		return new Location(x, y, clazz, id, angle);
	}
	// Persistable
	public String getName() {
		return "";
	}
	public String getType() {
		return "location";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("x", Integer.toString(x));
		res.put("y", Integer.toString(y));
		res.put("className", clazz);
		res.put("id", id);
		res.put("angle", Integer.toString(angle));
        res.put("hor", Double.toString(hor));
        res.put("ver", Double.toString(ver));
		return res;
	}
	public List<List<Persistable>> getChildren() {
		return new ArrayList<List<Persistable>>();
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		x = Integer.parseInt(attr.get("x"));
		y = Integer.parseInt(attr.get("y"));
		clazz = attr.get("className");
		id = attr.get("id");
		angle = attr.get("angle") != null ? Integer.parseInt(attr.get("angle")) : 0;
        hor = attr.get("hor") != null ? Double.parseDouble(attr.get("hor")) : 0.0;
        ver = attr.get("ver") != null ? Double.parseDouble(attr.get("ver")) : 0.0;
	}
	public void setChildren(List<List<Persistable>> childs) throws ModelException {
	}
	@Override
	public boolean equals(Object o) {
		if(o instanceof Location) {
			Location loc = (Location)o;
			if(x == loc.x && y == loc.y) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	@Override
	public int hashCode() {
		return (new Integer(x + y)).hashCode();
	}
	@Override
	public String toString() {
	    return "(" + x + "," + y + "," + clazz + "," + id + "," + angle + "," + hor + "," + ver + ")";
	}
}
