/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import dk.vajhoej.model.Duplicatable;
import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.PersistableBase;
import dk.vajhoej.model.SeparatePersistable;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.version.StringVersion;

/**
 * Class Model represents an architectural model.
 */
public class Model extends PersistableBase implements SeparatePersistable, Duplicatable, Serializable {
	private static final long serialVersionUID = 1L;
	private String name;
	private List<Box> boxes;
	private List<Connection> connections;
	private List<Picture> pictures;
	/**
     * Create instance of Model.
	 */
	public Model() {
		this("", new StringVersion(""));
	}
	/**
     * Create instance of Model.
	 * @param name name
	 * @param version version
	 */
	public Model(String name, Version version) {
		super(version);
		this.name = name;
		boxes = new ArrayList<Box>();
		connections = new ArrayList<Connection>();
		pictures = new ArrayList<Picture>();
	}
	/**
	 * Get name.
	 * @return name
	 */
	public String getName() {
		return name;
	}
	/**
	 * Set name.
	 * @param name name
	 */
	public void setName(String name) {
	    this.name = name;
	}
	/**
	 * Get boxes.
	 * @return unmodifiable list of boxes
	 */
	public List<Box> getBoxes() {
		return Collections.unmodifiableList(boxes);
	}
    /**
     * Add box.
     * @param box box
     */
    public void addBox(Box box) {
        boxes.add(box);
    }
    /**
     * Remove box.
     * @param box box
     */
    public void removeBox(Box box) {
        boxes.remove(box);
    }
	/**
	 * Get connections.
	 * @return unmodifiable list of connections
	 */
	public List<Connection> getConnections() {
		return Collections.unmodifiableList(connections);
	}
	/**
	 * Add connection.
	 * @param con connection
	 */
	public void addConnection(Connection con) {
		connections.add(con);
	}
	/**
	 * Remove connection.
	 * @param con connection
	 */
	public void removeConnection(Connection con) {
		connections.remove(con);
	}
	/**
	 * Get pictures.
	 * @return unmodifiable list of pictures
	 */
	public List<Picture> getPictures() {
		return Collections.unmodifiableList(pictures);
	}
	/**
	 * Add picture.
	 * @param pic picture
	 */
	public void addPicture(Picture pic) {
		pictures.add(pic);
	}
	/**
	 * Remove picture.
	 * @param pic picture
	 */
	public void removePicture(Picture pic) {
		pictures.remove(pic);
	}
	// Duplicatable
	public Model duplicate() {
		Model res = new Model(name, getVersion());
        for(Box b : boxes) {
            res.addBox(b.duplicate());
        }
        for(Connection c : connections) {
            res.addConnection(c.duplicate());
        }
		return res;
	}
	// Persistable:
    public String getId() {
        return name;
    }
	public String getType() {
		return "model";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("name", name);
		res.put("version", getVersion().toString());
		return res;
	}
	public List<List<Persistable>> getChildren() {
		List<Persistable> res0 = new ArrayList<Persistable>();
		res0.addAll(boxes);
		List<Persistable> res1 = new ArrayList<Persistable>();
		res1.addAll(connections);
		List<Persistable> res2 = new ArrayList<Persistable>();
		res2.addAll(pictures);
		List<List<Persistable>> res = new ArrayList<List<Persistable>>();
		res.add(res0);
		res.add(res1);
		res.add(res2);
		return res;
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		name = attr.get("name");
		setVersion(Version.parse(attr.get("version")));
	}
	public void setChildren(List<List<Persistable>> childs) {
		for(Persistable p : childs.get(0)) {
			boxes.add((Box)p);
		}
		for(Persistable p : childs.get(1)) {
			connections.add((Connection)p);
		}
		if(childs.size() > 2) {
			for(Persistable p : childs.get(2)) {
				pictures.add((Picture)p);
			}
		}
	}
	// basic
	@Override
	public boolean equals(Object o) {
		if(o instanceof Model) {
			Model m = (Model)o;
			if(name.equals(m.name) && getVersion().equals(m.getVersion())) {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	@Override
	public int hashCode() {
		return (name+getVersion()).hashCode();
	}
	@Override
	public String toString() {
	    return "(" + name + "," + boxes + "," + connections + ")";
	}
}
