/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.core;

import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.graphic.adv.BoxGraphicWrapper;
import dk.vajhoej.model.version.OneNumberVersion;

/**
 * Class ModelFilter translates a model to certain readonly models.
 */
public class ModelFilter  {
    /**
     * Hide hardware (CPU, memory and disk) information.
     * @param arg old model
     * @return new model
     */
    public static Model hideHardware(Model arg) {
        Model res = arg.duplicate();
        for(Box b : res.getBoxes()) {
            b.setCpu(0);
            b.setMem(0);
            b.setDisk(0);
        }
        return res;
    }
    private static void flattenComponents(Component c, Box b, Component parent) {
        while(!c.getComponents().isEmpty()) {
            flattenComponents(c.getComponents().get(0), b, c);
        }
        if(parent != null) {
            parent.removeComponent(c);
            b.addComponent(c);
        }
    }
    /**
     * Flatten component model so all components are at the outer/box level.
     * @param arg old model
     * @return new model
     */
    public static Model flattenComponents(Model arg) {
        Model res = arg.duplicate();
        for(Box b : res.getBoxes()) {
            int n = b.getComponents().size();
            for(int i = 0; i < n; i++) {
                flattenComponents(b.getComponents().get(i), b, null);
            }
        }
        return res;
    }
    /**
     * Hide connections.
     * @param arg old model
     * @return new model
     */
    public static Model hideConnections(Model arg) {
        Model res = arg.duplicate();
        while(!res.getConnections().isEmpty()) {
            res.removeConnection(res.getConnections().get(0));
        }
        return res;
    }
    /**
     * Remove duplicates.
     * @param arg old model
     * @return new model
     */
    public static Model removeDuplicates(Model arg) {
        Model res = arg.duplicate();
        List<Box> b = res.getBoxes();
        for(int i = b.size()-1; i >= 0; i--) {
            boolean already = false;
            for(int j = 0; j < i; j++) {
                if(b.get(i).getName().equals(b.get(j).getName())) {
                    already = true;
                }
            }
            if(already) {
                res.removeBox(b.get(i));
            }
        }
        return res;
    }
    /**
     * Ignore VM's and treat VM's like physical boxes.
     * @param arg old model
     * @return new model
     */
    public static Model ignoreVMs(Model arg) {
        Model res = arg.duplicate();
        int ix = 0;
        for(Box b : arg.getBoxes()) {
            if(b.getComponents().size() == 1 &&
               b.getComponents().get(0).getComponents().size() > 0 &&
               b.getComponents().get(0).getComponents().get(0).getComptype() == ComponentType.VM) {
                for(Component c : b.getComponents().get(0).getComponents()) {
                    ix++;
                    Box vb = new Box("VM #" + ix, new OneNumberVersion(1), "", "");
                    vb.setCpu(c.getCpu());
                    vb.setMem(c.getMem());
                    vb.setDisk(c.getDisk());
                    for(Component c2 : c.getComponents()) {
                        vb.addComponent(c2);
                    }
                    res.addBox(vb);
                }
                res.removeBox(b);
            }
        }
        return res;
    }
    /**
     * Filter non existing locations.
     * @param allloc all locations
     * @param m model
     * @return filtered locations
     */
    public static List<Location> filterLocations(List<Location> allloc, Model m) {
        List<Location> res = new ArrayList<Location>();
        for(Location loc : allloc) {
            boolean exists = false;
            for(Box b : m.getBoxes()) {
                if(loc.getId().equals(b.getId())) {
                    exists = true;
                }
            }
            for(Connection c : m.getConnections()) {
                if(loc.getId().equals(c.getId())) {
                    exists = true;
                }
            }
            if(exists) {
                res.add(loc);
            }
        }
        return res;
    }
    /**
     * Add locations for VM's.
     * @param allloc all locations
     * @param m model
     * @param w width
     * @param y0 start y 
     * @return added locations
     */
    public static List<Location> addLocations(List<Location> allloc, Model m, int w, int y0) {
        List<Location> res = new ArrayList<Location>(allloc);
        int ix = 0;
        for(Box b : m.getBoxes()) {
            if(b.getName().startsWith("VM #")) {
                b.setId(b.getName());
                res.add(new Location(ix % w, y0 + ix / w, BoxGraphicWrapper.class.getName(), b.getId()));
            }
        }
        return res;
    }
}
