/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;

import javax.swing.JPanel;

import dk.vajhoej.model.Util;
import dk.vajhoej.model.core.Component;
import dk.vajhoej.model.core.ComponentStatus;
import dk.vajhoej.model.graphic.base.ComponentGraphic;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class ComponentGraphicWrapper contains the added GUI functionality for
 * a ComponentGraphic instance. 
 */
public class ComponentGraphicWrapper extends ComponentGraphic implements CommonGraphicWrapper {
	private static final long serialVersionUID = 1L;
	private Component component;
	private static String label(Component c) {
		return (c.getVendor().equals("Open Source") ? "" : c.getVendor() + " ") + c.getName() + " " + c.getVersion();
	}
	private static String hardware(Component c) {
		StringBuilder sb = new StringBuilder();
		if(c.getCpu() > 0) {
			sb.append(Integer.toString(c.getCpu()));
			sb.append(" CPU cores");
		}
		if(c.getMem() > 0) {
			if(sb.length() > 0) sb.append(" ");
			sb.append(Util.intOrDoubleToString(c.getMem()));
			sb.append(" GB mem");
		}
		if(c.getDisk() > 0) {
			if(sb.length() > 0) sb.append(" ");
			sb.append(Util.intOrDoubleToString(c.getDisk()));
			sb.append(" TB disk");
		}
		return sb.length() > 0 ? sb.toString() : null;
	}
    /**
     * Create instance of ComponentGraphicWrapper.
     * @param cfg all graphic configurations
     * @param c component
     */
    public ComponentGraphicWrapper(final AllGraphicConfig cfg, Component c) {
        this(cfg, c, false);
    }
    /**
     * Create instance of ComponentGraphicWrapper.
     * @param cfg all graphic configurations
     * @param c component
     * @param gray gray out
     */
    public ComponentGraphicWrapper(final AllGraphicConfig cfg, Component c, boolean gray) {
		super(gray ? cfg.getComponentConfig(c.getVendor()).gray() : cfg.getComponentConfig(c.getVendor()),
		      c.getComponents().size(), label(c), hardware(c), c.getComptype());
		this.component = c;
		for(Component c2 : c.getComponents()) {
			add(new ComponentGraphicWrapper(cfg, c2, cfg.isGrayPassive() && (gray || c2.getStatus() == ComponentStatus.REMOTE_FAILOVER_PASSIVE)));
		}
		GUISupport.setupDropAndDrag(this, this, cfg);
		GUISupport.setupContextMenus(this, this, cfg);
	}
	// CommonGraphicWrapper:
	public boolean isReadonly() {
		return readonly;
	}
	public boolean acceptItem(Object item) {
		if(item instanceof Component) {
			return true;
		} else {
			return false;
		}
	}
	public void addItem(AllGraphicConfig cfg, Object item, Point p) {
		component.addComponent((Component)item);
		this.add(new ComponentGraphicWrapper(cfg, (Component)item));
		this.addChild();
		this.upAutoSize();
	}
	public void removeItem(CommonGraphicWrapper item) {
		component.removeComponent((Component)item.getRealObject());
		this.remove((JPanel)item);
		this.removeChild();
		this.upAutoSize();
	}
	public void updateState() {
		this.setLbl(label(component));
		this.setHw(hardware(component));
		this.setType(component.getComptype());
		this.upAutoSize();
	}
	public Component getRealObject() {
		return component;
	}
	public Transferable getTransferableObject() {
		return new ModelTransferable(component.duplicate());
	}
	public String getId() {
		return component.getName() + " " + component.getVersion().toString();
	}
	public boolean isRotating() {
		return false;
	}
	public void rotate(double angle, double hor, double ver) {
	}
    public double getAngle() {
        return 0.0;
    }
    public double getHor() {
        return 0.0;
    }
    public double getVer() {
        return 0.0;
    }
	public Object[] createNew() {
		return new Object[] { new Component() };
	}
}
