/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.TransferHandler;

import dk.vajhoej.model.Duplicatable;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.Util;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class GUISupport contains static utility methods to setup
 * context menus plus drop and drag.
 */
public class GUISupport {
	private static CommonGraphicWrapper del;
	/**
	 * Setup component for drag and drop.
	 * @param gui component as Swing component
	 * @param supp component as VSMA component
	 * @param cfg all graphic configurations
	 */
	public static void setupDropAndDrag(final JComponent gui, final CommonGraphicWrapper supp, final AllGraphicConfig cfg) {
		gui.setDropTarget(new DropTarget(gui, new DropTargetAdapter() {
			public void drop(DropTargetDropEvent dtde) {
				try {
					if(!supp.isReadonly()) {
						Transferable t  = dtde.getTransferable();
						if(t != null) {
							t = (Transferable)t.getTransferData(t.getTransferDataFlavors()[0]);
							if(t != null) {
								Object o = t.getTransferData(new ModelFlavor());
                                o = ((Duplicatable)o).duplicate();
								if(supp.acceptItem(o) && supp != del) {
									supp.addItem(cfg, o, dtde.getLocation());
									if(del != null) {
										CommonGraphicWrapper oldcontainer = (CommonGraphicWrapper)((JComponent)del).getParent();
										if(oldcontainer != null && (oldcontainer != gui.getParent() || oldcontainer instanceof ModelGraphicWrapper) && !oldcontainer.isReadonly())	 {
											oldcontainer.removeItem(del);
										}
									}
								}
							}
						}
					}
				} catch (UnsupportedFlavorException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}));
		gui.setTransferHandler(new TransferHandler("transferableObject"));
		gui.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent me) {
				if((me.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK) {
					JComponent jc = (JComponent)me.getSource();
					TransferHandler th = jc.getTransferHandler();
					if(gui.getParent() instanceof CommonGraphicWrapper) {
	                    CommonGraphicWrapper container = (CommonGraphicWrapper)gui.getParent();
	                    if(container.isReadonly())   {
	                        th.exportAsDrag(jc, me, TransferHandler.COPY);
	                    } else {
	                        th.exportAsDrag(jc, me, TransferHandler.COPY);
	                        del = supp;
	                    }
					}
				}
			}
		});
	}
    /**
     * Setup component with context menus.
     * @param gui component as Swing component
     * @param supp component as VSMA component
     * @param cfg all graphic configurations
     */
	public static void setupContextMenus(final JComponent gui, final CommonGraphicWrapper supp, final AllGraphicConfig cfg) {
		gui.addMouseListener(new MouseAdapter() {
			public void mousePressed(final MouseEvent me) {
				if((me.getModifiers() & MouseEvent.BUTTON1_MASK) == 0) {
					final CommonGraphicWrapper container = gui.getParent() instanceof CommonGraphicWrapper ? (CommonGraphicWrapper)gui.getParent() : null;
					JPopupMenu menu = new JPopupMenu();
					JMenu minew = new JMenu("New");
					Object[] objs = supp.createNew();
					if(supp.isReadonly() || objs.length <= 0) {
						minew.setEnabled(false);
					} else {
						for(final Object o : objs) {
							JMenuItem minewx = new JMenuItem(Util.toCap(((Persistable)o).getType()));
							minewx.addActionListener(new ActionListener() {
								public void actionPerformed(ActionEvent e) {
									if(supp.acceptItem(o)) {
										JDialog edit = new ObjectEditPanel(null, (Persistable)o);
										edit.setVisible(true);
										supp.addItem(cfg, o, me.getPoint());
									}
								}
							});
							minew.add(minewx);
						}
					}
					menu.add(minew);
					JMenuItem micopy = new JMenuItem("Copy");
					micopy.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							Clipboard cb = gui.getToolkit().getSystemClipboard();
							cb.setContents(supp.getTransferableObject(), null);
						}
					});
					menu.add(micopy);
					JMenuItem micut = new JMenuItem("Cut");
					if(container == null || container.isReadonly()) {
						micut.setEnabled(false);
					} else {
						micut.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								Clipboard cb = gui.getToolkit().getSystemClipboard();
								cb.setContents(supp.getTransferableObject(), null);
								container.removeItem(supp);
							}
						});
					}
					menu.add(micut);
					JMenuItem mipaste = new JMenuItem("Paste");
					if(supp.isReadonly()) {
						mipaste.setEnabled(false);
					} else {
						mipaste.addActionListener(new ActionListener() {
							public void actionPerformed(ActionEvent e) {
								try {
									Clipboard cb = gui.getToolkit().getSystemClipboard();
									Transferable t = cb.getContents(null);
									if(t != null) {
										Object o = t.getTransferData(new ModelFlavor());
										o = ((Duplicatable)o).duplicate();
										if(supp.acceptItem(o)) {
											supp.addItem(cfg, o, me.getPoint());
										}
									}
								} catch (UnsupportedFlavorException ex) {
									ex.printStackTrace();
								} catch (IOException ex) {
									ex.printStackTrace();
								}
							}
						});
					}
					menu.add(mipaste);
					JMenuItem miedit = new JMenuItem("Edit");
                    if(supp.isReadonly()) {
                        miedit.setEnabled(false);
                    } else {
    					miedit.addActionListener(new ActionListener() {
    						public void actionPerformed(ActionEvent e) {
    							JDialog edit = new ObjectEditPanel(null, (Persistable)supp.getRealObject());
    							edit.setVisible(true);
    							supp.updateState();
    						}
    					});
                    }
					menu.add(miedit);
					if(supp.isRotating()) {
						JMenu mirot = new JMenu("Rotate");
                        addRot(mirot, "180 degree counterclockwise", -Math.PI, supp);
                        addRot(mirot, "165 degree counterclockwise", -Math.PI*11/12, supp);
                        addRot(mirot, "150 degree counterclockwise", -Math.PI*5/6, supp);
                        addRot(mirot, "135 degree counterclockwise", -Math.PI*3/4, supp);
                        addRot(mirot, "120 degree counterclockwise", -Math.PI*2/3, supp);
                        addRot(mirot, "105 degree counterclockwise", -Math.PI*7/12, supp);
                        addRot(mirot, "90 degree counterclockwise", -Math.PI/2, supp);
                        addRot(mirot, "75 degree counterclockwise", -Math.PI*5/12, supp);
                        addRot(mirot, "60 degree counterclockwise", -Math.PI/3, supp);
                        addRot(mirot, "45 degree counterclockwise", -Math.PI/4, supp);
                        addRot(mirot, "30 degree counterclockwise", -Math.PI/6, supp);
                        addRot(mirot, "15 degree counterclockwise", -Math.PI/12, supp);
                        addRot(mirot, "0 degree", 0, supp);
                        addRot(mirot, "15 degree clockwise", Math.PI/12, supp);
                        addRot(mirot, "30 degree clockwise", Math.PI/6, supp);
                        addRot(mirot, "45 degree clockwise", Math.PI/4, supp);
                        addRot(mirot, "60 degree clockwise", Math.PI/3, supp);
                        addRot(mirot, "75 degree clockwise", Math.PI*5/12, supp);
                        addRot(mirot, "90 degree clockwise", Math.PI/2, supp);
                        addRot(mirot, "105 degree clockwise", Math.PI*7/12, supp);
                        addRot(mirot, "120 degree clockwise", Math.PI*2/3, supp);
                        addRot(mirot, "135 degree clockwise", Math.PI*3/4, supp);
                        addRot(mirot, "150 degree clockwise", Math.PI*5/6, supp);
                        addRot(mirot, "165 degree clockwise", Math.PI*11/12, supp);
                        addRot(mirot, "180 degree clockwise", Math.PI, supp);
						menu.add(mirot);
                        JMenu mimov = new JMenu("Move");
                        JMenuItem mimovup = new JMenuItem("Up");
                        mimovup.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                supp.rotate(supp.getAngle(), 0.0, -0.25);
                            }
                        });
                        mimov.add(mimovup);
                        JMenuItem mimovdown = new JMenuItem("Down");
                        mimovdown.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                supp.rotate(supp.getAngle(), 0.0, 0.25);
                            }
                        });
                        mimov.add(mimovdown);
                        JMenuItem mimovcenter = new JMenuItem("Center");
                        mimovcenter.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                supp.rotate(supp.getAngle(), 0.0, 0.0);
                            }
                        });
                        mimov.add(mimovcenter);
                        JMenuItem mimovleft = new JMenuItem("Left");
                        mimovleft.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                supp.rotate(supp.getAngle(), -0.25, 0.0);
                            }
                        });
                        mimov.add(mimovleft);
                        JMenuItem mimovright = new JMenuItem("Right");
                        mimovright.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                supp.rotate(supp.getAngle(), 0.25, 0.0);
                            }
                        });
                        mimov.add(mimovright);
                        menu.add(mimov);
					}
					menu.show(gui, me.getX(), me.getY());
				}
			}
            private void addRot(JMenu mirot, String label, final double angle, final CommonGraphicWrapper supp) {
                JMenuItem mirotx = new JMenuItem(label);
                mirotx.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        supp.rotate(angle, supp.getHor(), supp.getVer());
                    }
                });
                mirot.add(mirotx);
            }
		});
	}
}
