/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.awt.datatransfer.Transferable;

import javax.swing.JPanel;

import dk.vajhoej.model.core.Box;
import dk.vajhoej.model.core.Connection;
import dk.vajhoej.model.core.ExtModel;
import dk.vajhoej.model.core.Model;
import dk.vajhoej.model.core.Picture;
import dk.vajhoej.model.graphic.base.GraphicLayout;
import dk.vajhoej.model.graphic.base.ModelGraphic;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class ModelGraphicWrapper contains the added GUI functionality for
 * a ModelGraphic instance. 
 */
public class ModelGraphicWrapper extends ModelGraphic implements CommonGraphicWrapper {
	private static final long serialVersionUID = 1L;
	protected ExtModel mod;
	private static String label(Model m) {
		return m.getName() + " " + m.getVersion();
	}
	/**
	 * Add boxes and connections.
	 * @param cfg all graphic configurations
	 * @param m the model
	 */
	protected void addBoxesAndConnections(AllGraphicConfig cfg, ExtModel m) {
		for(Box b : m.getReal().getBoxes()) {
			add(new BoxGraphicWrapper(cfg, b));	
		}
		for(Connection c : m.getReal().getConnections()) {
			add(new ConnectionGraphicWrapper(cfg, c));	
		}
	}
	/**
	 * Add one new box.
	 * @param cfg all graphic configurations
	 * @param b box
	 * @param p the point to add it
	 */
	protected void addNewBox(AllGraphicConfig cfg, Box b, Point p) {
		this.add(new BoxGraphicWrapper(cfg, b));
		this.addChild();
		this.upAutoSize();
	}
    /**
     * Add one new connection.
     * @param cfg all graphic configurations
     * @param c connection
     * @param p the point to add it
     */
	protected void addNewConnection(AllGraphicConfig cfg, Connection c, Point p) {
		this.add(new ConnectionGraphicWrapper(cfg, c));
		this.addChild();
		this.upAutoSize();
	}
    /**
     * Add one new picture.
     * @param cfg all graphic configurations
     * @param pic picture
     * @param p the point to add it
     */
	protected void addNewPicture(AllGraphicConfig cfg, Picture pic, Point p) {
		this.add(new PictureGraphicWrapper(cfg, pic));
		this.addChild();
		this.upAutoSize();
	}
	/**
     * Create instance of ModelGraphicWrapper.
     * @param cfg all graphic configurations
     * @param m the model
	 * @param style layout style
	 */
	protected ModelGraphicWrapper(AllGraphicConfig cfg, ExtModel m, GraphicLayout style) {
		super(cfg.getModelConfig(), m.getReal().getBoxes().size() + m.getReal().getConnections().size(), label(m.getReal()), style);
		this.mod = m;
		addBoxesAndConnections(cfg, m);
		this.autoSize();
		GUISupport.setupDropAndDrag(this, this, cfg);
		GUISupport.setupContextMenus(this, this, cfg);
	}
	/**
	 * Create instance of ModelGraphicWrapper.
	 * @param cfg all graphic configurations
	 * @param m the model
	 */
	public ModelGraphicWrapper(final AllGraphicConfig cfg, ExtModel m) {
		this(cfg, m, GraphicLayout.SQUARE);
	}
	// CommonGraphicWrapper:
	public boolean isReadonly() {
		return readonly;
	}
	public boolean acceptItem(Object item) {
		if(item instanceof Box || item instanceof Connection || item instanceof Picture) {
			return true;
		} else {
			return false;
		}
	}
	public void addItem(AllGraphicConfig cfg, Object item, Point p) {
		if(item instanceof Box) {
			mod.getReal().addBox((Box)item);
			addNewBox(cfg, (Box)item, p);
		}
		if(item instanceof Connection) {
			mod.getReal().addConnection((Connection)item);
			addNewConnection(cfg, (Connection)item, p);
		}
		if(item instanceof Picture) {
			mod.getReal().addPicture((Picture)item);
			addNewPicture(cfg, (Picture)item, p);
		}
	}
	public void removeItem(CommonGraphicWrapper item) {
		Object o = item.getRealObject();
		if(o instanceof Box) {
			mod.getReal().removeBox((Box)o);
		}
		if(o instanceof Connection) {
			mod.getReal().removeConnection((Connection)o);
		}
		if(o instanceof Picture) {
			mod.getReal().removePicture((Picture)o);
		}
		this.remove((JPanel)item);
		this.removeChild();
		this.upAutoSize();
	}
	public void updateState() {
		this.setLbl(label(mod.getReal()));
		this.upAutoSize();
	}
	public Model getRealObject() {
		return mod.getReal();
	}
	public Transferable getTransferableObject() {
		return new ModelTransferable(mod.getReal().duplicate());
	}
	public String getId() {
		return mod.getReal().getName();
	}
	public boolean isRotating() {
		return false;
	}
	public void rotate(double angle, double hor, double ver) {
	}
    public double getAngle() {
        return 0.0;
    }
    public double getHor() {
        return 0.0;
    }
    public double getVer() {
        return 0.0;
    }
	public Object[] createNew() {
		return new Object[] { new Box(), new Connection(), new Picture() };
	}
}
