/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.adv;

import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.core.Box;
import dk.vajhoej.model.core.Connection;
import dk.vajhoej.model.core.ExtModel;
import dk.vajhoej.model.core.Location;
import dk.vajhoej.model.core.Picture;
import dk.vajhoej.model.graphic.base.GraphicLayout;
import dk.vajhoej.model.graphic.config.AllGraphicConfig;

/**
 * Class ModelGraphicWrapperAdv adds the extra functionality to
 * ModelGraphicWrapper to work with SuperSquare.
 */
public class ModelGraphicWrapperAdv extends ModelGraphicWrapper {
	private static final long serialVersionUID = 1L;
	private AllGraphicConfig cfg;
	@Override
	protected void addBoxesAndConnections(final AllGraphicConfig cfg, ExtModel m) {
		Grid g = this.calcGrid();
		for(int y = 0; y < g.getH(); y++) {
			for(int x = 0; x < g.getW(); x++) {
				if(m.getLocations() == null) {
					if(y * g.getW() + x < m.getReal().getBoxes().size()) {
						add(new BoxGraphicWrapper(cfg, m.getReal().getBoxes().get(y * g.getW() + x)));
					} else if(y * g.getW() + x < m.getReal().getBoxes().size() + m.getReal().getConnections().size()) {
						add(new ConnectionGraphicWrapper(cfg, m.getReal().getConnections().get(y * g.getW() + x - m.getReal().getBoxes().size())));
					} else if(y * g.getW() + x < m.getReal().getBoxes().size() + m.getReal().getConnections().size() + m.getReal().getPictures().size()) {
						add(new PictureGraphicWrapper(cfg, m.getReal().getPictures().get(y * g.getW() + x - m.getReal().getBoxes().size()  - m.getReal().getConnections().size())));
					} else {
						add(new EmptyGraphicWrapper(cfg));	
					}
				} else {
					boolean fnd = false;
					for(Location loc : m.getLocations()) {
						if(loc.getX() == x && loc.getY() == y) {
							for(Box b : m.getReal().getBoxes()) {
								if(loc.getClazz().equals(BoxGraphicWrapper.class.getName()) && loc.getId().equals(b.getId())) {
									add(new BoxGraphicWrapper(cfg, b));
									fnd = true;
									break;
								}
							}
							for(Connection c : m.getReal().getConnections()) {
								if(loc.getClazz().equals(ConnectionGraphicWrapper.class.getName()) && loc.getId().equals(c.getId())) {
									ConnectionGraphicWrapper cgw = new ConnectionGraphicWrapper(cfg, c);
									cgw.setAngle(loc.getAngle()*Math.PI/180);
                                    cgw.setVer(loc.getVer());
                                    cgw.setVer(loc.getHor());
									add(cgw);
									fnd = true;
									break;
								}
							}
							for(Picture p : m.getReal().getPictures()) {
								if(loc.getClazz().equals(PictureGraphicWrapper.class.getName()) && loc.getId().equals(p.getId())) {
									add(new PictureGraphicWrapper(cfg, p));
									fnd = true;
									break;
								}
							}
						}
					}
					if(!fnd) {
						add(new EmptyGraphicWrapper(cfg));	
					}
				}
			}
		}
	}
	@Override
	protected void addNewBox(final AllGraphicConfig cfg, Box b, Point p) {
		Grid g = this.calcGrid();
		int x = p.x / (getWidth() / g.getW());
		int y = p.y / (getHeight() / g.getH());
		int ix = y * g.getW() + x;
		this.remove(ix);
		this.add(new BoxGraphicWrapper(cfg, b), ix);
        mod.getLocations().add(new Location(x, y, BoxGraphicWrapper.class.getName(), b.getId()));
        mod.setLocations(getLocations());
		this.addChild();
		this.upAutoSize();
	}
	@Override
	protected void addNewConnection(final AllGraphicConfig cfg, Connection c, Point p) {
		Grid g = this.calcGrid();
		int x = p.x / (getWidth() / g.getW());
		int y = p.y / (getHeight() / g.getH());
		int ix = y * g.getW() + x;
		this.remove(ix);
		this.add(new ConnectionGraphicWrapper(cfg, c), ix);
        mod.getLocations().add(new Location(x, y, ConnectionGraphicWrapper.class.getClass().getName(), c.getId()));
        mod.setLocations(getLocations());
		this.addChild();
		this.upAutoSize();
	}
	@Override
	protected void addNewPicture(AllGraphicConfig cfg, Picture pic, Point p) {
		Grid g = this.calcGrid();
		int x = p.x / (getWidth() / g.getW());
		int y = p.y / (getHeight() / g.getH());
		int ix = y * g.getW() + x;
		this.remove(ix);
		this.add(new PictureGraphicWrapper(cfg, pic), ix);
        mod.getLocations().add(new Location(x, y, PictureGraphicWrapper.class.getClass().getName(), pic.getId()));
        mod.setLocations(getLocations());
		this.addChild();
		this.upAutoSize();
	}
	/**
	 * Create instance of ModelGraphicWrapperAdv.
	 * @param cfg all graphic configurations
	 * @param m the model
	 */
	public ModelGraphicWrapperAdv(AllGraphicConfig cfg, ExtModel m) {
		super(cfg, m, GraphicLayout.SUPER_SQUARE);
		this.cfg = cfg;
	}
	/**
	 * Get location of all boxes and connections.
	 * @return location list
	 */
	public List<Location> getLocations() {
		List<Location> res = new ArrayList<Location>();
		Grid g = this.calcGrid();
		int boxid = 0;
		int conid = 0;
		int picid = 0;
		for(int y = 0; y < g.getH(); y++) {
			for(int x = 0; x < g.getW(); x++) {
				int ix = y * g.getW() + x;
				CommonGraphicWrapper cgw = (CommonGraphicWrapper)this.getComponent(ix);
				if(cgw instanceof BoxGraphicWrapper) {
				    boxid++;
				    Box b = (Box)cgw.getRealObject();
                    b.setId(boxid);
					res.add(new Location(x, y, cgw.getClass().getName(), b.getId()));
					
				}
				if(cgw instanceof ConnectionGraphicWrapper) {
					double angle = ((ConnectionGraphicWrapper)cgw).getAngle() * 180 / Math.PI;
                    double ver = ((ConnectionGraphicWrapper)cgw).getVer();
                    double hor = ((ConnectionGraphicWrapper)cgw).getHor();
                    conid++;
                    Connection c = (Connection)cgw.getRealObject();
                    c.setId(conid);
					res.add(new Location(x, y, cgw.getClass().getName(), c.getId(), (int)Math.round(angle), ver, hor));
				}
				if(cgw instanceof PictureGraphicWrapper) {
                    picid++;
                    Picture p = (Picture)cgw.getRealObject();
                    p.setId(picid);
					res.add(new Location(x, y, cgw.getClass().getName(), p.getId()));
				}
			}
		}
		return res;
	}
	@Override
	public void removeItem(CommonGraphicWrapper item) {
		int ix = -1;
	    for(int i = 0; i < this.getComponentCount(); i++) {
			if(this.getComponent(i) == item) {
				ix = i;
			}
		}
		this.add(new EmptyGraphicWrapper(cfg), ix);
		super.removeItem(item);
	}
}
