/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.base;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.GridLayout;

import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class BaseGraphic is the base class for all graphic objects.
 */
public abstract class BaseGraphic extends JPanel {
	private static final long serialVersionUID = 1L;
	protected GraphicConfig cfg;
	protected int nchild;
	protected GraphicLayout style;
	protected boolean fixed;
	protected int w = 0;
	protected int h = 0;
	protected boolean readonly;
	/**
	 * Class Grid represents width and height.
	 */
    protected static class Grid {
        private int w;
        private int h;
        /**
         * Create instance of Grid.
         * @param w width
         * @param h height
         */
        public Grid(int w, int h) {
            super();
            this.w = w;
            this.h = h;
        }
        /**
         * Get width.
         * @return width
         */
        public int getW() {
            return w;
        }
        /**
         * Get height.
         * @return height
         */
        public int getH() {
            return h;
        }
    }
	/**
     * Create instance of BaseGraphic with dynamic size.
     * @param cfg graphic configuration
	 * @param nchild number children
	 * @param style layout style
	 * @param nlbls number labels
     * @param uml UML style
	 */
	protected BaseGraphic(GraphicConfig cfg, int nchild, GraphicLayout style, int nlbls, boolean uml) {
		this.cfg = cfg;
		this.nchild = nchild;
		this.style = style;
		this.fixed = false;
        this.readonly = false;
        setBorder(nlbls, uml);
	}
	/**
     * Create instance of BaseGraphic with fixed size.
     * @param cfg graphic configuration
	 * @param w width
	 * @param h height
     * @param nchild number children
     * @param style layout style
     * @param nlbls number labels
     * @param uml UML style
	 */
	protected BaseGraphic(GraphicConfig cfg, int w, int h, int nchild, GraphicLayout style, int nlbls, boolean uml) {
        this.cfg = cfg;
        this.nchild = nchild;
        this.style = style;
		fixed = true;
		this.w = w;
		this.h = h;
        this.readonly = false;
        setBorder(nlbls, uml);
	}
	/**
	 * Set readonly status.
	 * @param readonly readonly status
	 */
	public void setReadonly(boolean readonly) {
	    this.readonly = readonly;
	}
    /**
     * Auto size this and children.
     */
    public void autoSize() {
        Dimension dim = calcSize();
        this.setPreferredSize(dim);
        this.setMinimumSize(dim);
        this.setSize(dim);
        if(nchild > 0) {
            Grid g = calcGrid();
            this.setLayout(new GridLayout(g.getH(), g.getW()));
            for(Component c : this.getComponents()) {
                if(c instanceof BaseGraphic) {
                    ((BaseGraphic)c).autoSize();
                }
            }
        }
    }
    /**
     * Tell parent to upAutoSize and if no parent do autoSize.
     */
    public void upAutoSize() {
        if(getParent() instanceof BaseGraphic) {
            ((BaseGraphic)getParent()).upAutoSize();
        } else {
            autoSize();
            getParent().getParent().validate();
        }
    }
	/**
	 * Set border.
	 * @param nlbls number labels
	 */
    protected void setBorder(int nlbls, boolean uml) {
        this.setBorder(new EmptyBorder(cfg.getMargin()+(1+nlbls)*cfg.getFont().getSize(),
                                       cfg.getMargin()+(uml?4*cfg.getMargin():0),
                                       cfg.getMargin(),
                                       cfg.getMargin()));
    }
    /**
     * Add child.
     */
	protected void addChild() {
		nchild++;
	}
	/**
	 * Remove child.
	 */
	protected void removeChild() {
		nchild--;
	}
	/**
	 * Calculate size.
	 * @return dimension
	 */
	protected Dimension calcSize() {
		if(!fixed) {
			w = 2 * cfg.getMargin();
			h = 2 * cfg.getMargin();
			if(nchild > 0) {
				int w1 = 0;
				int h1 = 0;
				for(Component c : this.getComponents()) {
					if(c instanceof BaseGraphic) {
						Dimension dim = ((BaseGraphic)c).calcSize();
						w1 = Math.max(w1, dim.width);
						h1 = Math.max(h1, dim.height);
					}
				}
				Grid g = calcGrid();
		    	w += g.getW()*w1;
		    	h += g.getH()*h1;
			}
		}
		return new Dimension(w, h);
	}
    /**
     * Draw rectangle.
     * @param g2 graphics context
     */
	protected void drawRectangle(Graphics2D g2) {
		drawRectangle(g2, 0, 0, 0);
	}
    /**
     * Draw rectangle behind.
     * @param g2 graphics context
     */
	protected void drawRectangleBehind(Graphics2D g2) {
        drawRectangle(g2, cfg.getMargin()*2/3, cfg.getMargin()*2/3, 0);
	}
    /**
     * Draw UML component.
     * @param g2 graphics context
     */
	protected void drawUML(Graphics2D g2) {
	    drawRectangle(g2, 0, 0, 2*cfg.getMargin());
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawRect(cfg.getMargin(),
                5*getHeight()/16 - cfg.getMargin(),
                4*cfg.getMargin(),
                2*cfg.getMargin());
        g2.drawRect(cfg.getMargin(),
                11*getHeight()/16 - cfg.getMargin(),
                4*cfg.getMargin(),
                2*cfg.getMargin());
        g2.setColor(cfg.getFillColor());
        g2.fillRect(cfg.getMargin() + cfg.getThickness()/2,
                5*getHeight()/16 - cfg.getMargin() + cfg.getThickness()/2,
                4*cfg.getMargin() - cfg.getThickness(),
                2*cfg.getMargin() - cfg.getThickness());
        g2.fillRect(cfg.getMargin() + cfg.getThickness()/2,
                11*getHeight()/16 - cfg.getMargin() + cfg.getThickness()/2,
                4*cfg.getMargin() - cfg.getThickness(),
                2*cfg.getMargin() - cfg.getThickness());
	}
	/**
	 * Draw circle.
     * @param g2 graphics context
	 */
	protected void drawCircle(Graphics2D g2) {
	    drawCircle(g2, 0);
	}
    /**
     * Draw circle behind.
     * @param g2 graphics context
     */
    protected void drawCircleBehind(Graphics2D g2) {
        drawCircle(g2, cfg.getMargin()*2/3);
    }
	/**
	 * Draw barrel.
     * @param g2 graphics context
	 */
	protected void drawBarrel(Graphics2D g2) {
		drawBarrel(g2, 0);
	}
	/**
	 * Draw barrel behind.
     * @param g2 graphics context
	 */
	protected void drawBarrelBehind(Graphics2D g2) {
		drawBarrel(g2, cfg.getMargin()*2/3);
	}
    /**
     * Draw firewall.
     * @param g2 graphics context
     */
    protected void drawFirewall(Graphics2D g2) {
        drawFirewall(g2, 0);
    }
    /**
     * Draw firewall behind.
     * @param g2 graphics context
     */
    protected void drawFirewallBehind(Graphics2D g2) {
        drawFirewall(g2, cfg.getMargin()*2/3);
    }
	/**
	 * Draw arrow.
     * @param g2 graphics context
     * @param offset vertical offset
     * @param angle angle
	 */
	protected void drawArrow(Graphics2D g2, int offset, double angle) {
		g2.setColor(cfg.getLineColor());
		g2.setStroke(new BasicStroke(cfg.getThickness()));
        int adjw = getWidth() - (int)Math.min(getWidth()/Math.abs(Math.cos(angle)), getHeight()/Math.abs(Math.sin(angle)));
        if(Math.cos(angle) != 0 && Math.sin(angle) != 0) {
            adjw += 2*cfg.getMargin();
        }
		int[] x = { 2*cfg.getMargin() + adjw/2,
				    getWidth() - 2*cfg.getMargin() - adjw/2,
				    getWidth() - cfg.getMargin() - adjw/2,
				    getWidth() - 2*cfg.getMargin() - adjw/2,
				    2*cfg.getMargin() + adjw/2,
				    cfg.getMargin() + adjw/2};
		int[] y = { getHeight()/2 - cfg.getMargin() + offset,
				    getHeight()/2 - cfg.getMargin() + offset,
				    getHeight()/2 + offset,
				    getHeight()/2 + cfg.getMargin() + offset,
				    getHeight()/2 + cfg.getMargin() + offset,
				    getHeight()/2  + offset};
        int[] x2 = new int[x.length];
        int[] y2 = new int[y.length];
        adjustPolyInnerFill(offset, x, y, x2, y2);
		g2.drawPolygon(x, y, x.length);
		g2.setColor(cfg.getFillColor());
		g2.fillPolygon(x2, y2, x2.length);
	}
    /**
     * Draw left bus.
     * @param g2 graphics context
     */
    protected void drawLeftBus(Graphics2D g2, double angle) {
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawLine(2*cfg.getMargin(),
                    getHeight()/2 - cfg.getMargin(),
                    getWidth(),
                    getHeight()/2 - cfg.getMargin());
        g2.drawLine(2*cfg.getMargin(),
                    getHeight()/2 + cfg.getMargin(),
                    getWidth(),
                    getHeight()/2 + cfg.getMargin());
        g2.drawLine(cfg.getMargin(),
                    getHeight()/2,
                    2*cfg.getMargin(),
                    getHeight()/2 - cfg.getMargin());
        g2.drawLine(cfg.getMargin(),
                    getHeight()/2,
                    2*cfg.getMargin(),
                    getHeight()/2 + cfg.getMargin());
        g2.setColor(cfg.getFillColor());
        int[] x = { cfg.getMargin(),
                    2*cfg.getMargin(),
                    getWidth(),
                    getWidth(),
                    2*cfg.getMargin() };
        int[] y = { getHeight()/2,
                    getHeight()/2 - cfg.getMargin(),
                    getHeight()/2 - cfg.getMargin(),
                    getHeight()/2 + cfg.getMargin(),
                    getHeight()/2 + cfg.getMargin() };
        int[] x2 = new int[x.length];
        int[] y2 = new int[y.length];
        adjustPolyInnerFill(0, x, y, x2, y2);
        g2.fillPolygon(x2, y2, x2.length);
    }
    /**
     * Draw middle bus.
     * @param g2 graphics context
     */
    protected void drawMidBus(Graphics2D g2, double angle) {
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawLine(0,
                    getHeight()/2 - cfg.getMargin(),
                    getWidth(),
                    getHeight()/2 - cfg.getMargin());
        g2.drawLine(0,
                    getHeight()/2 + cfg.getMargin(),
                    getWidth(),
                    getHeight()/2 + cfg.getMargin());
        g2.setColor(cfg.getFillColor());
        g2.fillRect(0,
                    getHeight()/2 - cfg.getMargin() + cfg.getThickness()/2,
                    getWidth(),
                    2*cfg.getMargin() - cfg.getThickness());
    }
    /**
     * Draw right bus.
     * @param g2 graphics context
     */
    protected void drawRightBus(Graphics2D g2, double angle) {
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawLine(0,
                    getHeight()/2 - cfg.getMargin(),
                    getWidth() - 2*cfg.getMargin(),
                    getHeight()/2 - cfg.getMargin());
        g2.drawLine(0,
                    getHeight()/2 + cfg.getMargin(),
                    getWidth() - 2*cfg.getMargin(),
                    getHeight()/2 + cfg.getMargin());
        g2.drawLine(getWidth() - cfg.getMargin(),
                    getHeight()/2,
                    getWidth() - 2*cfg.getMargin(),
                    getHeight()/2 - cfg.getMargin());
        g2.drawLine(getWidth() - cfg.getMargin(),
                    getHeight()/2,
                    getWidth() - 2*cfg.getMargin(),
                    getHeight()/2 + cfg.getMargin());
        g2.setColor(cfg.getFillColor());
        int[] x = { 0,
                    getWidth() - 2*cfg.getMargin(),
                    getWidth() - cfg.getMargin(),
                    getWidth() - 2*cfg.getMargin(),
                    0 };
        int[] y = { getHeight()/2 - cfg.getMargin(),
                    getHeight()/2 - cfg.getMargin(),
                    getHeight()/2,
                    getHeight()/2 + cfg.getMargin(),
                    getHeight()/2 + cfg.getMargin() };
        int[] x2 = new int[x.length];
        int[] y2 = new int[y.length];
        adjustPolyInnerFill(0, x, y, x2, y2);
        g2.fillPolygon(x2, y2, x2.length);
    }
    /**
     * Draw middle cross.
     * @param g2 graphics context
     */
    protected void drawMidCross(Graphics2D g2, double angle) {
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawLine(0,
                    getHeight()/2 - cfg.getMargin(),
                    getWidth(),
                    getHeight()/2 - cfg.getMargin());
        g2.drawLine(0,
                    getHeight()/2 + cfg.getMargin(),
                    getWidth(),
                    getHeight()/2 + cfg.getMargin());
        g2.drawLine(getWidth()/2 - cfg.getMargin(),
                    0,
                    getWidth()/2 - cfg.getMargin(),
                    getHeight());
        g2.drawLine(getWidth()/2 + cfg.getMargin(),
                    0,
                    getWidth()/2 + cfg.getMargin(),
                    getHeight());
        g2.setColor(cfg.getFillColor());
        g2.fillRect(0,
                    getHeight()/2 - cfg.getMargin() + cfg.getThickness()/2,
                    getWidth(),
                    2*cfg.getMargin() - cfg.getThickness());
        g2.fillRect(getWidth()/2 - cfg.getMargin() + cfg.getThickness()/2,
                    0,
                    2*cfg.getMargin() - cfg.getThickness(),
                    getHeight());
    }
    /**
     * Draw end cross.
     * @param g2 graphics context
     */
    protected void drawEndCross(Graphics2D g2, double angle) {
        int w = getWidth();
        int h = getHeight();
        if(Math.abs(Math.cos(angle)) > 2 * Math.abs(Math.sin(angle))) {
            // nothing
        } else if(Math.abs(Math.cos(angle)) < 0.5 * Math.abs(Math.sin(angle))) {
            int tmp;
            tmp = w;
            w = h;
            h = tmp;
        } else {
            int tmp = Math.min(w, h);
            w = tmp;
            h = tmp;
        }
        int adjw = w - (int)Math.min(w/Math.abs(Math.cos(angle)), h/Math.abs(Math.sin(angle)));
        if(Math.cos(angle) != 0 && Math.sin(angle) != 0) {
            adjw += 2*cfg.getMargin();
        }
        int[] x = { 2*cfg.getMargin() + adjw/2,
                    w/2 - cfg.getMargin(),
                    w/2 - cfg.getMargin(),
                    w/2,
                    w/2 + cfg.getMargin(),
                    w/2 + cfg.getMargin(),
                    w - 2*cfg.getMargin() - adjw/2,
                    w - cfg.getMargin() - adjw/2,
                    w - 2*cfg.getMargin() - adjw/2,
                    w/2 + cfg.getMargin(),
                    w/2 + cfg.getMargin(),
                    w/2,
                    w/2 - cfg.getMargin(),
                    w/2 - cfg.getMargin(),
                    2*cfg.getMargin() + adjw/2,
                    cfg.getMargin() + adjw/2};
        int[] y = { h/2 - cfg.getMargin(),
                    h/2 - cfg.getMargin(),
                    2*cfg.getMargin() + adjw/2,
                    cfg.getMargin() + adjw/2,
                    2*cfg.getMargin() + adjw/2,
                    h/2 - cfg.getMargin(),
                    h/2 - cfg.getMargin(),
                    h/2,
                    h/2 + cfg.getMargin(),
                    h/2 + cfg.getMargin(),
                    h - 2*cfg.getMargin() - adjw/2,
                    h - cfg.getMargin() - adjw/2,
                    h - 2*cfg.getMargin() - adjw/2,
                    h/2 + cfg.getMargin(),
                    h/2 + cfg.getMargin(),
                    h/2};
        int[] x2 = new int[x.length];
        int[] y2 = new int[y.length];
        adjustPolyInnerFill(0, x, y, x2, y2);
        g2.setColor(cfg.getFillColor());
        g2.fillPolygon(x2, y2, x2.length);
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawPolygon(x, y, x.length);
    }
	/**
	 * Draw string at top.
	 * @param s string
     * @param g2 graphics context
	 */
	protected void drawTopString(String s, Graphics2D g2) {
		g2.setColor(cfg.getTextColor());
		g2.setFont(cfg.getFont());
		g2.drawString(s,
				      getWidth()/2 - g2.getFontMetrics().stringWidth(s)/2,
				      g2.getFontMetrics().getHeight()+cfg.getMargin());
	}
	/**
	 * Draw string just below top.
	 * @param s string
     * @param g2 graphics context
	 */
	protected void drawBelowTopString(String s, Graphics2D g2) {
		g2.setColor(cfg.getTextColor());
		g2.setFont(cfg.getFont());
		g2.drawString(s,
				      getWidth()/2 - g2.getFontMetrics().stringWidth(s)/2,
				      2*g2.getFontMetrics().getHeight()+cfg.getMargin());
	}
	/**
	 * Draw string at middle.
	 * @param s string
     * @param g2 graphics context
	 */
	protected void drawMiddleString(String s, Graphics2D g2) {
        drawOffsetMiddleString(s, g2, 0);
	}
	/**
	 * Draw string just below middle.
	 * @param s string
     * @param g2 graphics context
	 */
	protected void drawBelowMiddleString(String s, Graphics2D g2) {
        drawOffsetMiddleString(s, g2, g2.getFontMetrics().getHeight());
	}
	protected void drawOffsetMiddleString(String s, Graphics2D g2, int offset) {
        g2.setColor(cfg.getTextColor());
        g2.setFont(cfg.getFont());
        g2.drawString(s,
                      getWidth()/2 - g2.getFontMetrics().stringWidth(s)/2,
                      getHeight()/2 + g2.getFontMetrics().getHeight()/2 + offset);
	}
	/**
	 * Draw string at bottom.
	 * @param s string
     * @param g2 graphics context
	 */
	protected void drawBottomString(String s, Graphics2D g2) {
		g2.setColor(cfg.getTextColor());
		g2.setFont(cfg.getFont());
		g2.drawString(s,
				      getWidth()/2 - g2.getFontMetrics().stringWidth(s)/2,
				      getHeight() - cfg.getMargin());
	}
	/**
	 * Add label to dimensions.
	 * @param dim input dimension
	 * @param lbl label
	 * @return output dimension
	 */
	protected Dimension addLabel(Dimension dim, String lbl) {
		return new Dimension(Math.max(dim.width, 4*cfg.getMargin() + lbl.length()*cfg.getFont().getSize()/2),
                             dim.height + 2*cfg.getFont().getSize());
	}
	/**
     * Add two labels to dimensions.
     * @param dim input dimension
     * @param lbl label
	 * @param xtra extra label
	 * @return output dimension
	 */
	protected Dimension addTwoLabels(Dimension dim, String lbl, String xtra) {
		return new Dimension(Math.max(dim.width, 4*cfg.getMargin() + Math.max(lbl.length(), xtra.length())*cfg.getFont().getSize()/2),
                             dim.height + 4*cfg.getFont().getSize());
	}
	/**
	 * Calculate grid (number of components vertical and horizontal).
	 * @return grid
	 */
	protected Grid calcGrid() {
		int n = (int)Math.ceil(Math.sqrt(nchild));
		int nw;
		int nh;
    	switch(style) {
	    	case SQUARE:
	    		nw = n;
	    		nh = n;
	    		if(depth() % 4 != 0) {
                    while((nw-1)*nh >= nchild) nw--;
	    		} else {
                    while(nw*(nh-1) >= nchild) nh--;
	    		}
	    		break;
	    	case SUPER_SQUARE:
	    		nw = SuperSquare.getInstance().getDimX();
	    		nh = SuperSquare.getInstance().getDimY();
	    		break;
	    	case VERTICAL:
	    		nw = 1;
	    		nh = nchild;
	    		break;
	    	case HORIZONTAL:
	    		nw = nchild;
	    		nh = 1;
	    		break;
	    	default:
	    		throw new ModelRuntimeException("Unknown style");
    	}
    	return new Grid(nw, nh);
	}
    private void drawRectangle(Graphics2D g2, int hoffset, int voffset, int extraleft) {
        int adjw = (getWidth() > 4*getHeight() && calcSize().width < getWidth() - 4*getHeight()) ? (getWidth() - 4*getHeight()) : 0;
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        if(!cfg.isRound()) {
            g2.drawRect(cfg.getMargin() + hoffset + adjw/2 + extraleft,
                        cfg.getMargin() + voffset,
                        getWidth()-2*cfg.getMargin() - adjw - extraleft,
                        getHeight()-2*cfg.getMargin());
        } else {
            g2.drawRoundRect(cfg.getMargin() + hoffset + adjw/2 + extraleft,
                             cfg.getMargin() + voffset,
                             getWidth()-2*cfg.getMargin() - adjw - extraleft,
                             getHeight()-2*cfg.getMargin(),
                             4*cfg.getMargin(),
                             4*cfg.getMargin());
        }
        g2.setColor(cfg.getFillColor());
        if(!cfg.isRound()) {
            g2.fillRect(cfg.getMargin() + cfg.getThickness()/2 + hoffset + adjw/2 + extraleft,
                        cfg.getMargin() + cfg.getThickness()/2 + voffset,
                        getWidth() - 2*cfg.getMargin() - cfg.getThickness() - adjw - extraleft,
                        getHeight() - 2*cfg.getMargin() - cfg.getThickness());
        } else {
            g2.fillRoundRect(cfg.getMargin() + cfg.getThickness()/2 + hoffset + adjw/2 + extraleft,
                             cfg.getMargin() + cfg.getThickness()/2 + voffset,
                             getWidth() - 2*cfg.getMargin() - cfg.getThickness() - adjw - extraleft,
                             getHeight() - 2*cfg.getMargin() - cfg.getThickness(),
                             4*cfg.getMargin(),
                             4*cfg.getMargin());
        }
    }
    private void drawCircle(Graphics2D g2, int offset) {
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawOval(cfg.getMargin() + offset,
                    cfg.getMargin() + offset,
                    getWidth()-2*cfg.getMargin(),
                    getHeight()-2*cfg.getMargin());
        g2.setColor(cfg.getFillColor());
        g2.fillOval(cfg.getMargin() + cfg.getThickness()/2 + offset,
                    cfg.getMargin() + cfg.getThickness()/2 + offset,
                    getWidth() - 2*cfg.getMargin() - cfg.getThickness(),
                    getHeight() - 2*cfg.getMargin() - cfg.getThickness());
        
    }
    private void drawFirewall(Graphics2D g2, int offset) {
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawRect(cfg.getMargin() + offset,
                    cfg.getMargin() + offset,
                    getWidth()-2*cfg.getMargin(),
                    getHeight()-2*cfg.getMargin() - g2.getFontMetrics().getHeight()*3/2);
        g2.setColor(Color.ORANGE);
        g2.fillRect(cfg.getMargin() + cfg.getThickness()/2 + offset,
                    cfg.getMargin() + cfg.getThickness()/2 + offset,
                    getWidth() - 2*cfg.getMargin() - cfg.getThickness(),
                    getHeight() - 2*cfg.getMargin() - g2.getFontMetrics().getHeight()*3/2 - cfg.getThickness());
        g2.setColor(Color.WHITE);
        g2.drawLine(cfg.getMargin() + cfg.getThickness() + offset,
                    getHeight()/8 + offset,
                    getWidth() - cfg.getMargin() - cfg.getThickness() + offset,
                    getHeight()/8 + offset);
        g2.drawLine(cfg.getMargin() + cfg.getThickness() + offset,
                    getHeight()/4 + offset,
                    getWidth() - cfg.getMargin() - cfg.getThickness() + offset,
                    getHeight()/4 + offset);
        g2.drawLine(cfg.getMargin() + cfg.getThickness() + offset,
                    getHeight()*3/8 + offset,
                    getWidth() - cfg.getMargin() - cfg.getThickness() + offset,
                    getHeight()*3/8 + offset);
        g2.drawLine(cfg.getMargin() + cfg.getThickness() + offset,
                    getHeight()/2 + offset,
                    getWidth() - cfg.getMargin() - cfg.getThickness() + offset,
                    getHeight()/2 + offset);
        g2.drawLine(cfg.getMargin() + cfg.getThickness() + offset,
                    getHeight()*5/8 + offset,
                    getWidth() - cfg.getMargin() - cfg.getThickness() + offset,
                    getHeight()*5/8 + offset);
        g2.drawLine(cfg.getMargin() + cfg.getThickness() + offset,
                    getHeight()*3/4 + offset,
                    getWidth() - cfg.getMargin() - cfg.getThickness() + offset,
                    getHeight()*3/4 + offset);
        g2.drawLine(cfg.getMargin() + cfg.getThickness()/2 + offset,
                    getHeight()*7/8 + offset,
                    getWidth() - cfg.getMargin() - cfg.getThickness() + offset,
                    getHeight()*7/8 + offset);
        g2.drawLine(getWidth()/4 + offset,
                    cfg.getMargin() + cfg.getThickness() + offset,
                    getWidth()/4 + offset,
                    getHeight() - cfg.getMargin() - g2.getFontMetrics().getHeight()*3/2 - cfg.getThickness() + offset);
        g2.drawLine(getWidth()/2 + offset,
                    cfg.getMargin() + cfg.getThickness() + offset,
                    getWidth()/2 + offset,
                    getHeight() - cfg.getMargin() - g2.getFontMetrics().getHeight()*3/2 - cfg.getThickness() + offset);
        g2.drawLine(getWidth()*3/4 + offset,
                    cfg.getMargin() + cfg.getThickness() + offset,
                    getWidth()*3/4 + offset,
                    getHeight() - cfg.getMargin() - g2.getFontMetrics().getHeight()*3/2 - cfg.getThickness() + offset);
    }
    private void drawBarrel(Graphics2D g2, int offset) {
        int adjw = (getWidth() > 4*getHeight() && calcSize().width < 4*getHeight()) ? (getWidth() - 4*getHeight()) : 0;
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawOval(2*cfg.getMargin() + offset + adjw/2,
                    getHeight()*2/3 + offset,
                    getWidth() - 4*cfg.getMargin() - adjw,
                    getHeight()/6);
        g2.setColor(cfg.getFillColor());
        g2.fillOval(2*cfg.getMargin() + cfg.getThickness()/2 + offset + adjw/2,
                    getHeight()*2/3 + cfg.getThickness()/2 + offset,
                    getWidth() - 4*cfg.getMargin() - cfg.getThickness() - adjw,
                    getHeight()/6 - cfg.getThickness());
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawLine(2*cfg.getMargin() + offset + adjw/2,
                    getHeight()/4 + offset,
                    2*cfg.getMargin() + offset + adjw/2,
                    getHeight()*3/4 + offset);
        g2.drawLine(getWidth() - 2*cfg.getMargin() + offset - adjw/2,
                    getHeight()/4 + offset,
                    getWidth() - 2*cfg.getMargin() + offset - adjw/2,
                    getHeight()*3/4 + offset);
        g2.setColor(cfg.getFillColor());
        g2.fillRect(2*cfg.getMargin() + cfg.getThickness()/2 + offset + adjw/2,
                    getHeight()/4 + offset,
                    getWidth() - 4*cfg.getMargin() - cfg.getThickness() - adjw,
                    getHeight()/2);
        g2.setColor(cfg.getLineColor());
        g2.setStroke(new BasicStroke(cfg.getThickness()));
        g2.drawOval(2*cfg.getMargin() + offset + adjw/2,
                    getHeight()/6 + offset,
                    getWidth() - 4*cfg.getMargin() - adjw,
                    getHeight()/6);
        g2.setColor(cfg.getFillColor());
        g2.fillOval(2*cfg.getMargin() + cfg.getThickness()/2 + offset + adjw/2,
                    getHeight()/6 + cfg.getThickness()/2 + offset,
                    getWidth() - 4*cfg.getMargin() - cfg.getThickness() - adjw,
                    getHeight()/6 - cfg.getThickness());
    }
    private void adjustPolyInnerFill(int offset, int[] x, int[] y, int[] x2, int[] y2) {
        for(int i = 0; i < x.length; i++) {
            if(x[i] < getWidth()/2) {
                x2[i] = x[i] + cfg.getThickness()/2;
            } else if(x[i] > getWidth()/2) {
                x2[i] = x[i] - cfg.getThickness()/2;
            } else {
                x2[i] = x[i];
            }
            if(y[i] < getHeight()/2 + offset) {
                y2[i] = y[i] + cfg.getThickness()/2;
            } else if(y[i] > getHeight()/2 + offset) {
                y2[i] = y[i] - cfg.getThickness()/2;
            } else {
                y2[i] = y[i];
            }
        }
    }
    private int depth() {
        int res = 0;
        JPanel p = this;
        do {
            res++;
            p = (JPanel)p.getParent();
        } while(p != null && p instanceof BaseGraphic);
        return res;
    }
}
