/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.base;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

import dk.vajhoej.model.core.BoxType;
import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class BoxGraphic is the graphic object for a hardware box.
 */
public class BoxGraphic extends BaseGraphic {
	private static final long serialVersionUID = 1L;
	private String lbl;
	private String hw;
	private BoxType type;
	/**
     * Create instance of BoxGraphic with dynamic size.
     * @param cfg graphic configuration
     * @param nchild number of children
     * @param lbl label
     * @param hw hardware
     * @param type box type
	 */
	public BoxGraphic(GraphicConfig cfg, int nchild, String lbl, String hw, BoxType type) {
		super(cfg, nchild, GraphicLayout.SQUARE, hw != null ? 2 : 1, false);
		this.lbl = lbl;
		this.hw = hw;
		this.type = type;
	}
	/**
	 * 
     * Create instance of BoxGraphic with fixed size.
     * @param cfg graphic configuration
	 * @param w width
	 * @param h height
     * @param nchild number of children
     * @param lbl label
     * @param hw hardware
     * @param type box type
	 */
	public BoxGraphic(GraphicConfig cfg, int w, int h, int nchild, String lbl, String hw, BoxType type) {
		super(cfg, w, h, nchild, GraphicLayout.SQUARE, hw != null ? 2 : 1, false);
		this.lbl = lbl;
		this.hw = hw;
		this.type = type;
	}
	/**
	 * Set label.
	 * @param lbl label
	 */
	protected void setLbl(String lbl) {
		this.lbl = lbl;
	}
	/**
	 * Set hardware.
	 * @param hw hardware
	 */
	protected void setHw(String hw) {
		this.hw = hw;
		this.setBorder(hw != null ? 2 : 1, false);
	}
	/**
	 * Set box type.
	 * @param type box type
	 */
	protected void setType(BoxType type) {
		this.type = type;
	}
	@Override
	protected Dimension calcSize() {
		if(hw != null) {
			return addTwoLabels(super.calcSize(), lbl,  hw);
		} else {
			return addLabel(super.calcSize(), lbl);
		}
	}
	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		switch(type) {
			case SINGLE_SYS:
				drawRectangle(g2);
				drawTopString(lbl, g2);
				if(hw != null) {
					drawBelowTopString(hw, g2);
				}
				break;
			case MULTI_SYS:
				drawRectangleBehind(g2);
				drawRectangle(g2);
				drawTopString(lbl, g2);
				if(hw != null) {
					drawBelowTopString(hw, g2);
				}
				break;
			case SINGLE_DISK:
				drawBarrel(g2);
				drawMiddleString(lbl, g2);
				if(hw != null) {
					drawBelowMiddleString(hw, g2);
				}
				break;
			case MULTI_DISK:
				drawBarrelBehind(g2);
				drawBarrel(g2);
				drawMiddleString(lbl, g2);
				if(hw != null) {
					drawBelowMiddleString(hw, g2);
				}
				break;
            case SINGLE_FIREWALL:
                drawFirewall(g2);
                drawBottomString(lbl, g2);
                break;
            case MULTI_FIREWALL:
                drawFirewallBehind(g2);
                drawFirewall(g2);
                drawBottomString(lbl, g2);
                break;
            case SINGLE_LOADBALANCER:
                drawCircle(g2);
                drawMiddleString(lbl, g2);
                break;
            case MULTI_LOADBALANCER:
                drawCircleBehind(g2);
                drawCircle(g2);
                drawMiddleString(lbl, g2);
                break;
			default:
				break;
		}
    }
}
