/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.base;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;

import dk.vajhoej.model.core.ConnectionType;
import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class ConnectionGraphic is the graphic object for a connection
 * between hardware boxes.
 */
public class ConnectionGraphic extends BaseGraphic {
	private static final long serialVersionUID = 1L;
	private String lbl;
	private ConnectionType conntype;
	private double angle;
	private double hor;
	private double ver;
	/**
     * Create instance of connection graphic with dynamic size.
     * @param cfg graphic configuration
     * @param lbl label
	 */
	public ConnectionGraphic(GraphicConfig cfg, String lbl, ConnectionType conntype) {
		super(cfg, 0, GraphicLayout.SQUARE, 0, false);
		this.lbl = lbl;
        this.conntype = conntype;
        this.angle = 0;
        this.hor = 0.0;
        this.ver = 0.0;
	}
	/**
     * Create instance of connection graphic with fixed size.
     * @param cfg graphic configuration
	 * @param w width
	 * @param h height
     * @param lbl label
	 */
	public ConnectionGraphic(GraphicConfig cfg, int w, int h, String lbl, ConnectionType conntype) {
		super(cfg, w, h, 0, GraphicLayout.SQUARE, 0, false);
		this.lbl = lbl;
        this.conntype = conntype;
        this.angle = 0;
        this.hor = 0.0;
        this.ver = 0.0;
	}
	/**
	 * Set label.
	 * @param lbl label
	 */
	protected void setLbl(String lbl) {
		this.lbl = lbl;
	}
	/**
	 * Set connection type.
	 * @param conntype connection type
	 */
	protected void setConntype(ConnectionType conntype) {
	    this.conntype = conntype;
	}
	/**
	 * Get angle.
	 * @return angle.
	 */
	public double getAngle() {
		return angle;
	}
	/**
	 * Set angle.
	 * @param angle angle
	 */
	public void setAngle(double angle) {
		this.angle = angle;
	}
    /**
     * Get relative horizontal move.
     * @return horizontal move
     */
    public double getHor() {
        return hor;
    }
    /**
     * Set relative horizontal move.
     * @param hor horizontal move
     */
    public void setHor(double hor) {
        this.hor = hor;
    }
    /**
     * Get relative vertical move.
     * @return vertical move
     */
    public double getVer() {
        return ver;
    }
    /**
     * Set relative vertical move.
     * @param ver vertical move
     */
    public void setVer(double ver) {
        this.ver = ver;
    }
	@Override
	protected Dimension calcSize() {
		Dimension orig = addLabel(super.calcSize(), "***" + lbl + "***");
		Dimension adj = new Dimension((int)Math.abs(orig.width * Math.cos(angle)),
				                      (int)Math.max(orig.height, Math.abs(orig.width * Math.sin(angle))));
		return adj;
	}
	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		String[] lblparts = lbl.split("&");
        g2.transform(AffineTransform.getTranslateInstance(hor*getWidth(), ver*getHeight()));
        g2.transform(AffineTransform.getRotateInstance(angle, getWidth()/2, getHeight()/2));
		switch(conntype) {
    		case SINGLE:
                drawArrow(g2, 0, angle);
                drawMiddleString(lbl, g2);
    		    break;
    		case DOUBLE:
                drawArrow(g2, -3*cfg.getMargin(), angle);
                drawOffsetMiddleString(lblparts[0].trim(), g2, -3*cfg.getMargin());
                drawArrow(g2, 3*cfg.getMargin(), angle);
                drawOffsetMiddleString(lblparts.length > 1 ? lblparts[1].trim() : "", g2, 3*cfg.getMargin());
    		    break;
    		case TRIPPLE:
                drawArrow(g2, -6*cfg.getMargin(), angle);
                drawOffsetMiddleString(lblparts[0], g2, -6*cfg.getMargin());
                drawArrow(g2, 0, angle);
                drawOffsetMiddleString(lblparts.length > 1 ? lblparts[1].trim() : "", g2, 0);
                drawArrow(g2, 6*cfg.getMargin(), angle);
                drawOffsetMiddleString(lblparts.length > 2 ? lblparts[2].trim() : "", g2, 6*cfg.getMargin());
    		    break;
    		case LEFTBUS:
                drawLeftBus(g2, angle);
                drawMiddleString(lbl, g2);
    		    break;
    		case MIDBUS:
                drawMidBus(g2, angle);
                drawMiddleString(lbl, g2);
    		    break;
            case RIGHTBUS:
                drawRightBus(g2, angle);
                drawMiddleString(lbl, g2);
                break;
            case MIDCROSS:
                drawMidCross(g2, angle);
                drawMiddleString(lbl, g2);
                break;
            case ENDCROSS:
                drawEndCross(g2, angle);
                drawMiddleString(lbl, g2);
                break;
		}
    }
}
