/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.base;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.JPanel;

import dk.vajhoej.model.graphic.config.GraphicConfig;

/**
 * Class PaletteGraphic is the base class for all palette graphic objects.
 */
public abstract class PaletteGraphic extends BaseGraphic {
	private static final long serialVersionUID = 1L;
	private String lbl;
	private boolean show;
	/**
     * Create instance of PaletteGraphic with dynamic size.
     * @param cfg graphic configuration
	 * @param nchild number children
	 * @param lbl label
     * @param show show palette
	 * @param layout graphic layout
	 */
	public PaletteGraphic(GraphicConfig cfg, int nchild, String lbl, boolean show, GraphicLayout layout) {
		super(cfg, nchild, layout, 1, false);
		this.lbl = lbl;
        this.show = show;
        readonly = true;
	}
	/**
     * Create instance of PaletteGraphic with fixed size.
     * @param cfg graphic configuration
	 * @param w width
	 * @param h height
     * @param nchild number children
     * @param lbl label
     * @param show show palette
     * @param layout graphic layout
	 */
	public PaletteGraphic(GraphicConfig cfg, int w, int h, int nchild, String lbl, boolean show, GraphicLayout layout) {
		super(cfg, w, h, nchild, layout, 1, false);
		this.lbl = lbl;
        this.show = show;
        readonly = true;
	}
	/**
	 * Is show.
	 * @return show
	 */
	public boolean isShow() {
		return show;
	}
	/**
	 * Set show.
	 * @param show show
	 */
	public void setShow(boolean show) {
		this.show = show;
		this.upAutoSize();
	}
	/**
	 * Swap two items in palette.
	 * @param ix1 index first item
	 * @param ix2 index second item
	 */
	public void swap(int ix1, int ix2) {
        JPanel p = (JPanel)this.getParent();
        int ixa = Math.min(ix1, ix2);
        int ixb = Math.max(ix1, ix2);
        Component ca = p.getComponent(ixa);
        Component cb = p.getComponent(ixb);
        p.remove(ixb);
        p.remove(ixa);
        p.add(cb, ixa);
        p.add(ca, ixb);
        p.getParent().getParent().validate();
	}
	@Override
	protected Dimension calcSize() {
		if(show) {
			return addLabel(super.calcSize(), lbl);
		} else {
			return addTwoLabels(new Dimension(0, 0), lbl, "...");
		}
	}
	@Override
	public void paintChildren(Graphics g) {
		if(show) {
			super.paintChildren(g);
		}
	}
	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		g2.setColor(Color.WHITE);
		g2.fillRect(0, 0, getWidth(), getHeight());
		drawTopString(lbl, g2);
		if(!show) {
			g2.setColor(cfg.getLineColor());
			g2.fillRect(getWidth()/4 - cfg.getMargin()/2,
					    2*g2.getFontMetrics().getHeight()+cfg.getMargin()/2,
					    cfg.getMargin(),
					    cfg.getMargin());
			g2.fillRect(getWidth()*3/8 - cfg.getMargin()/2,
			            2*g2.getFontMetrics().getHeight()+cfg.getMargin()/2,
			            cfg.getMargin(),
			            cfg.getMargin());
			g2.fillRect(getWidth()/2 - cfg.getMargin()/2,
			            2*g2.getFontMetrics().getHeight()+cfg.getMargin()/2,
			            cfg.getMargin(),
			            cfg.getMargin());
			g2.fillRect(getWidth()*5/8 - cfg.getMargin()/2,
			            2*g2.getFontMetrics().getHeight()+cfg.getMargin()/2,
			            cfg.getMargin(),
			            cfg.getMargin());
			g2.fillRect(getWidth()*3/4 - cfg.getMargin()/2,
				        2*g2.getFontMetrics().getHeight()+cfg.getMargin()/2,
				        cfg.getMargin(),
				        cfg.getMargin());
		}
    }
}
