/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.config;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;
import dk.vajhoej.model.SeparatePersistable;

/**
 * Class AllGraphicConfig contains alle graphical configurations.
 */
public class AllGraphicConfig implements SeparatePersistable, Cloneable {
	private static final String MODEL = "MODEL";
	private static final String PALETTE = "PALETTE";
	private static final String BOX_PREFIX = "BOX_";
	private static final String COMPONENT_PREFIX = "COMPONENT_";
	private static final String CONNECTION = "CONNECTION";
	private static final String STANDARD = "STANDARD";
	private static final String BOX_UNKNOWN = BOX_PREFIX + "unknown";
	private static final String COMPONENT_UNKNOWN = COMPONENT_PREFIX + "unknown";
	private Map<String, GraphicConfig> cfgs;
	private boolean grayPassive;
	private String store;
	/**
	 * Create instance of AllGraphicConfig.
	 */
	public AllGraphicConfig() {
		cfgs = new HashMap<String, GraphicConfig>();
		grayPassive = false;
	}
	/**
	 * Get standard config.
	 * @return standard config
	 */
	public GraphicConfig getStandardConfig() {
		return cfgs.get(STANDARD);
	}
	/**
	 * Get model config.
	 * @return model config
	 */
	public GraphicConfig getModelConfig() {
		if(cfgs.containsKey(MODEL)) {
			return cfgs.get(MODEL);
		} else {
			return cfgs.get(STANDARD);
		}
	}
	/**
	 * Get palette config.
	 * @return palette config
	 */
	public GraphicConfig getPaletteConfig() {
		if(cfgs.containsKey(PALETTE)) {
			return cfgs.get(PALETTE);
		} else {
			return cfgs.get(STANDARD);
		}
	}
	/**
	 * Get box config.
	 * @param vendor vendor
	 * @return box config
	 */
	public GraphicConfig getBoxConfig(String vendor) {
		if(cfgs.containsKey(BOX_PREFIX + vendor)) {
			return cfgs.get(BOX_PREFIX + vendor);
		} else if(cfgs.containsKey(BOX_UNKNOWN)) {
			return cfgs.get(BOX_UNKNOWN);
		} else {
			return cfgs.get(STANDARD);
		}
	}
	/**
	 * Get component config.
	 * @param vendor vendor
	 * @return component config
	 */
	public GraphicConfig getComponentConfig(String vendor) {
		if(cfgs.containsKey(COMPONENT_PREFIX + vendor)) {
			return cfgs.get(COMPONENT_PREFIX + vendor);
		} else if(cfgs.containsKey(COMPONENT_UNKNOWN)) {
			return cfgs.get(COMPONENT_UNKNOWN);
		} else {
			return cfgs.get(STANDARD);
		}
	}
	/**
	 * Get connection config.
	 * @return connection config
	 */
	public GraphicConfig getConnectionConfig() {
		if(cfgs.containsKey(CONNECTION)) {
			return cfgs.get(CONNECTION);
		} else {
			return cfgs.get(STANDARD);
		}
	}
	/**
	 * Set standard config.
	 * @param cfg standard config
	 */
	public void setStandardConfig(GraphicConfig cfg) {
		cfgs.put(STANDARD, cfg);
	}
	/**
	 * Set model config.
	 * @param cfg model config
	 */
	public void setModelConfig(GraphicConfig cfg) {
		cfgs.put(MODEL, cfg);
	}
	/**
	 * Set palette config.
	 * @param cfg palette config
	 */
	public void setPaletteConfig(GraphicConfig cfg) {
		cfgs.put(PALETTE, cfg);
	}
	/**
	 * Set box config.
	 * @param vendor vendor
	 * @param cfg box config
	 */
	public void setBoxConfig(String vendor, GraphicConfig cfg) {
		cfgs.put(BOX_PREFIX + vendor, cfg);
	}
	/**
	 * Set component config.
	 * @param vendor vendor
	 * @param cfg component config
	 */
	public void setComponentConfig(String vendor, GraphicConfig cfg) {
		cfgs.put(COMPONENT_PREFIX + vendor, cfg);
	}
	/**
	 * Set connection config.
	 * @param cfg connection config
	 */
	public void setConnectionConfig(GraphicConfig cfg) {
		cfgs.put(CONNECTION, cfg);
	}
	/**
	 * Is gray passive set.
	 * @return gray passive
	 */
	public boolean isGrayPassive() {
	    return grayPassive;
	}
	/**
	 * Set gray passive.
	 * @param grayPassive gray passive
	 */
	public void setGrayPassive(boolean grayPassive) {
	    this.grayPassive = grayPassive;
	}
    /**
     * Get list of box vendor names.
     * @return unmodifiable list of box vendor names
     */
	public List<String> getBoxes() {
		List<String> res = new ArrayList<String>();
		for(String k : cfgs.keySet()) {
			if(k.startsWith(BOX_PREFIX)) {
				res.add(k.substring(BOX_PREFIX.length()));
			}
		}
		return Collections.unmodifiableList(res);
	}
    /**
     * Get list of component vendor names.
     * @return unmodifiable list of component vendor names
     */
	public List<String> getComponents() {
		List<String> res = new ArrayList<String>();
		for(String k : cfgs.keySet()) {
			if(k.startsWith(COMPONENT_PREFIX)) {
				res.add(k.substring(COMPONENT_PREFIX.length()));
			}
		}
		return Collections.unmodifiableList(res);
	}
    // Cloneable:
    @Override
    public AllGraphicConfig clone() {
        AllGraphicConfig res = new AllGraphicConfig();
        res.cfgs = cfgs;
        res.grayPassive = grayPassive;
        return res;
    }
	// Persistable
	public String getId() {
		return "Single";
	}
	public String getType() {
		return "allGraphicsConfig";
	}
	public String getStore() {
		return store;
	}
	public void setStore(String store) {
		this.store = store;
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("grayPassive", Boolean.toString(grayPassive));
		return res;
	}
	public List<List<Persistable>> getChildren() {
		List<List<Persistable>> res = new ArrayList<List<Persistable>>();
		List<Persistable> res0 = new ArrayList<Persistable>();
		for(String k : cfgs.keySet()) {
			res0.add(new ExtGraphicConfig(k, cfgs.get(k)));
		}
		res.add(res0);
		return res;
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
	    grayPassive = attr.get("grayPassive") != null ? Boolean.parseBoolean(attr.get("grayPassive")) : false;
	}
	public void setChildren(List<List<Persistable>> childs)	throws ModelException {
		for(Persistable p : childs.get(0)) {
			ExtGraphicConfig cfg = (ExtGraphicConfig)p;
			cfgs.put(cfg.getKey(), cfg.getConfig());
		}
	}
	// basic
	@Override
	public String toString() {
	    return "(" + cfgs + "," + store + ")";
	}
}
