/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.graphic.config;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.List;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.NameValuePairList;
import dk.vajhoej.model.Persistable;

/**
 * Class GraphicConfig contains a single graphical configuration.
 */
public class GraphicConfig implements Persistable, Cloneable {
	public static final Color PASTEL_YELLOW = new Color(255, 255, 205);
	public static final Color PASTEL_GREEN = new Color(240, 255, 240);
	public static final Color PASTEL_BLUE = new Color(240, 255, 255);
	public static final Color PASTEL_LAVENDER = new Color(230, 230, 255);
	public static final Color PASTEL_RED = new Color(255, 240, 205);
	public static final Color WHITE_LIGHT_GRAY = new Color((Color.LIGHT_GRAY.getRed() + Color.WHITE.getRed())/2,
			                                               (Color.LIGHT_GRAY.getGreen() + Color.WHITE.getGreen())/2,
			                                               (Color.LIGHT_GRAY.getBlue() + Color.WHITE.getBlue())/2);
	private int margin;
	private int thickness;
	private Font font;
	private Color textColor;
	private Color lineColor;
	private Color fillColor;
	private boolean round;
	/**
     * Create instance of GraphicConfig.
	 */
	public GraphicConfig() {
		this(0, 0, null, null, null, null);
	}
	/**
     * Create instance of GraphicConfig.
	 * @param margin margin
	 * @param thickness line thickness
	 * @param font text font
	 * @param textColor text color
	 * @param lineColor line color
	 * @param fillColor fill color
	 */
    public GraphicConfig(int margin, int thickness, Font font, Color textColor, Color lineColor, Color fillColor) {
        this(margin, thickness, font, textColor, lineColor, fillColor, false);
    }
    /**
     * Create instance of GraphicConfig.
     * @param margin margin
     * @param thickness line thickness
     * @param font text font
     * @param textColor text color
     * @param lineColor line color
     * @param fillColor fill color
     * @param round round corners
     */
	public GraphicConfig(int margin, int thickness, Font font, Color textColor, Color lineColor, Color fillColor, boolean round) {
		super();
		this.margin = margin;
		this.thickness = thickness;
		this.font = font;
		this.textColor = textColor;
		this.lineColor = lineColor;
		this.fillColor = fillColor;
		this.round = round;
	}
	/**
	 * Get margin.
	 * @return margin
	 */
	public int getMargin() {
		return margin;
	}
	/**
	 * Set margin.
	 * @param margin margin
	 */
	public void setMargin(int margin) {
		this.margin = margin;
	}
	/**
	 * Get line thickness.
	 * @return line thickness
	 */
	public int getThickness() {
		return thickness;
	}
	/**
	 * Set line thickness.
	 * @param thickness line thickness
	 */
	public void setThickness(int thickness) {
		this.thickness = thickness;
	}
	/**
	 * Get text font.
	 * @return text font
	 */
	public Font getFont() {
		return font;
	}
	/**
	 * Set text font.
	 * @param font text font
	 */
	public void setFont(Font font) {
		this.font = font;
	}
	/**
	 * Get text color.
	 * @return text color
	 */
	public Color getTextColor() {
		return textColor;
	}
	/**
	 * Set text color.
	 * @param textColor text color
	 */
	public void setTextColor(Color textColor) {
		this.textColor = textColor;
	}
	/**
	 * Get line color.
	 * @return line color
	 */
	public Color getLineColor() {
		return lineColor;
	}
	/**
	 * Set line color.
	 * @param lineColor line color
	 */
	public void setLineColor(Color lineColor) {
		this.lineColor = lineColor;
	}
	/**
	 * Get fill color.
	 * @return fill color
	 */
	public Color getFillColor() {
		return fillColor;
	}
	/**
	 * Set fill color.
	 * @param fillColor fill color
	 */
	public void setFillColor(Color fillColor) {
		this.fillColor = fillColor;
	}
	/**
	 * Is round corners.
	 * @return round corners
	 */
	public boolean isRound() {
	    return round;
	}
	/**
	 * Set round corners.
	 * @param round round corners
	 */
	public void setRound(boolean round) {
	    this.round = round;
	}
	/**
	 * Gray out.
	 * @return new GraphicCnfig
	 */
	public GraphicConfig gray() {
	    GraphicConfig res = (GraphicConfig)clone(); 
        res.setTextColor(Color.GRAY);
        res.setLineColor(Color.GRAY);
	    res.setFillColor(WHITE_LIGHT_GRAY);
	    return res;
	}
	// Cloneable:
	@Override
	public Object clone() {
		return new GraphicConfig(margin, thickness, font, textColor, lineColor, fillColor, round);
	}
	// Persistable:
	public String getId() {
		return Integer.toString(hashCode());
	}
	public String getType() {
		return "graphicsConfig";
	}
	public NameValuePairList getAttributes() {
		NameValuePairList res = new NameValuePairList();
		res.put("margin", Integer.toString(margin));
		res.put("thickness", Integer.toString(thickness));
		res.put("font", font.getFontName() + "-" + font.getSize());
		res.put("textColor", Integer.toHexString(textColor.getRGB()).substring(2));
		res.put("lineColor", Integer.toHexString(lineColor.getRGB()).substring(2));
		res.put("fillColor", Integer.toHexString(fillColor.getRGB()).substring(2));
		res.put("round", Boolean.toString(round));
		return res;
	}
	public List<List<Persistable>> getChildren() {
		return new ArrayList<List<Persistable>>();
	}
	public void setAttributes(NameValuePairList attr) throws ModelException {
		margin = Integer.parseInt(attr.get("margin"));
		thickness = Integer.parseInt(attr.get("thickness"));
		font = Font.decode(attr.get("font"));
		textColor = Color.decode("0x" + attr.get("textColor"));
		lineColor = Color.decode("0x" + attr.get("lineColor"));
		fillColor = Color.decode("0x" + attr.get("fillColor"));
		round = attr.get("round") != null ? Boolean.parseBoolean(attr.get("round")) : false;
	}
	public void setChildren(List<List<Persistable>> childs)	throws ModelException {
	}
	// basic
	@Override
	public String toString() {
	    return "(" + margin + "," + thickness + "," + font + "," + textColor + "," + lineColor + "," + fillColor + "," + round + ")";
	}
}
