/*
 * Copyright 2010 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.model.test;

import org.junit.Test;

import dk.vajhoej.model.ModelException;
import dk.vajhoej.model.ModelRuntimeException;
import dk.vajhoej.model.Version;
import dk.vajhoej.model.VersionFactory;
import static org.junit.Assert.*;

public class TestVersion {
	private static class FoobarVersion extends Version {
		private static final long serialVersionUID = 1L;
		private int major;
		private int minor;
		public FoobarVersion(int major, int minor) {
			this.major = major;
			this.minor = minor;
		}
		public int getMajor() {
			return major;
		}
		public int getMinor() {
			return minor;
		}
		@Override
		public int compareTo(Version v) {
			if(v instanceof FoobarVersion) {
				FoobarVersion v2 = (FoobarVersion)v;
				if(major == v2.major) {
					return minor - v2.minor;
				} else {
					return major - v2.major;
				}
			} else {
				throw new ModelRuntimeException("Comparing incompatible version numbers");
			}
		}
		@Override
		public String toString() {
			return "(" + major + "." + minor + ")";
		}
		@Override
		public FoobarVersion bumpLeast() {
			return new FoobarVersion(major, minor+1);
		}
		@Override
		public FoobarVersion bumpMost() {
			return new FoobarVersion(major+1, 0);
		}
		@Override
		public FoobarVersion bumpSecondLeast() {
			return bumpMost();
		}
		@Override
		public FoobarVersion bumpSecondMost() {
			return bumpLeast();
		}
	}
	private static class FoobarVersionFactory implements VersionFactory {
		static {
			Version.registerFactory(new FoobarVersionFactory());
		}
		public FoobarVersion create(String s) {
			if(s.length() >= 2 &&
			   s.charAt(0) == '(' &&
			   s.charAt(s.length()-1) == ')') {
				String[] parts = s.substring(1, s.length() -1).split("\\.");
				if(parts.length == 2) {
					return new FoobarVersion(Integer.parseInt(parts[0]), Integer.parseInt(parts[1]));
				} else {
					return null;
				}
			} else {
				return null;
			}
		}
	}
	@Test
	public void testBasic() {
		FoobarVersion ver = new FoobarVersion(1,2);
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
	}
	@Test
	public void testFactory() {
		FoobarVersionFactory fact = new FoobarVersionFactory();
		FoobarVersion ver = fact.create("(1.2)");
		assertEquals("Major", 1, ver.getMajor());
		assertEquals("Minor", 2, ver.getMinor());
	}
	@Test
	public void testIndirect() {
		try {
			FoobarVersion ver = (FoobarVersion)Version.parse("(1.2)");
			assertEquals("Major", 1, ver.getMajor());
			assertEquals("Minor", 2, ver.getMinor());
		} catch (ModelException e) {
			fail(e.getMessage());
		}
	}
}
